import { Component, OnInit } from '@angular/core';
import { JhaThemingService } from '@jha/jharesponsive/jha-theme';

// Needed for jackHenry integration
declare var jackHenry: any;
declare var xperience: any;

@Component({
	selector: 'app-root',
	templateUrl: './app.component.html',
	styleUrls: ['./app.component.scss'],
	providers: [JhaThemingService]
})
export class AppComponent implements OnInit {

	constructor(
		private themingService: JhaThemingService
	) {
	}

	ngOnInit(): void {

		this.themingService.ThemePath = './assets/themes';

		// Subscribe to needed message bus items
		jackHenry.xperience.messageBus.subscribe('ThemeChng', this.onThemeChng);
		jackHenry.xperience.messageBus.subscribe('AppCls', this.onAppCls);

		// onReady, run onFrameworkReady to connect to bus
		jackHenry.xperience.messageBus.onReady(this.onFrameworkReady, 10000);
	}

	// Handle called when the Xperience framework is ready
	private onFrameworkReady = () => {
		// Make sure we are hosted before doing any connections
		if (jackHenry.xperience.isFrameworkHosted) {
			// Must connect with the message bus before sending any messages
			jackHenry.xperience.messageBus.connect();
			if (jackHenry.xperience.messageBus.isConnected) {
				// Use startup arguments to load the current theme to match Xperience
				this.themingService.LoadThemeColor(jackHenry.xperience.startupArguments.ColorType);
			}
		}
	}

	// Handle theme change message from Xperience
	public onThemeChng = (messageContext: any) => {
		const message = messageContext.messageToJson();

		this.themingService.LoadThemeColor(message.ColorType);
	}

	// Handle app close message from Xperience
	private onAppCls(messageContext: any) {
		const message = messageContext.messageToJson();
		const response = new jackHenry.enterprise.businessObjects.AppClsResponse();
		response.ClsStat = 'ClsAck';
		response.RsStat = 'Success';

		messageContext.reply(response);
		jackHenry.xperience.messageBus.disconnect(true);
		window.close();
	}
}
