import { NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { HttpClientModule } from '@angular/common/http';
import { RouterModule, Routes } from '@angular/router';
import { LocationStrategy, HashLocationStrategy } from '@angular/common';
import { enableProdMode } from '@angular/core';

// Responsive UI

import { JhaButtonsModule} from '@jha/jharesponsive/jha-buttons';
import { JhaDialogBoxModule } from '@jha/jharesponsive/jha-dialog-box';
import { JhaFloatingToolbarModule } from '@jha/jharesponsive/jha-floating-toolbar';
import { JhaFormsModule } from '@jha/jharesponsive/jha-forms';
import { JhaHeaderModule } from '@jha/jharesponsive/jha-header';
import { JhaIconModule } from '@jha/jharesponsive/jha-icon';
import { JhaLayoutModule } from '@jha/jharesponsive/jha-layout';
import { JhaMasterDetailModule } from '@jha/jharesponsive/jha-master-detail';
import { JhaMegaMenuModule } from '@jha/jharesponsive/jha-mega-menu';
import { JhaNavModule } from '@jha/jharesponsive/jha-nav';
import { JhaNotificationsModule } from '@jha/jharesponsive/jha-notifications';
import { JhaPasswordStrengthMeterModule } from '@jha/jharesponsive/jha-password-strength-meter';
import { JhaPercentageCircleModule } from '@jha/jharesponsive/jha-percentage-circle';
import { JhaPipesModule } from '@jha/jharesponsive/jha-pipes';
import { JhaProcessTimelineModule } from '@jha/jharesponsive/jha-process-timeline';
import { JhaProgressBarModule } from '@jha/jharesponsive/jha-progress-bar';
import { JhaRatingModule } from '@jha/jharesponsive/jha-rating';
import { JhaRecordDetailModule } from '@jha/jharesponsive/jha-record-detail';
import { JhaResponsiveCoreModule } from '@jha/jharesponsive/jha-responsive-core';
import { JhaTileModule } from '@jha/jharesponsive/jha-tile';
import { JhaToggleModule } from '@jha/jharesponsive/jha-toggle';
import { JhaUnsavedChangesModule } from '@jha/jharesponsive/jha-unsaved-changes';


// Third Party Modules
import * as $ from 'jquery';
import * as bootstrap from 'bootstrap';
import { ButtonsModule } from 'ngx-bootstrap/buttons';
import { PaginationModule } from 'ngx-bootstrap/pagination';
import { TypeaheadModule } from 'ngx-bootstrap/typeahead';
import { ContextMenuModule } from 'primeng/contextmenu';
import { CheckboxModule } from 'primeng/checkbox';
import { MultiSelectModule } from 'primeng/multiselect';
import { RadioButtonModule } from 'primeng/radiobutton';
import { StepsModule } from 'primeng/steps';
import { TableModule } from 'primeng/table';
import { TreeTableModule } from 'primeng/treetable';
import { InputMaskModule } from 'primeng/inputmask';
import { PickListModule } from 'primeng/picklist';
import { InputNumberModule } from 'primeng/inputnumber';

import { MatLegacyChipsModule } from '@angular/material/legacy-chips';
import { MatLegacySelectModule } from '@angular/material/legacy-select';
import { MatLegacyTabsModule } from '@angular/material/legacy-tabs';

import { AngularSplitModule } from 'angular-split';

// Services
import { LogInOutService } from './services/log-in-out/log-in-out.service';
import { LoggedInGuard } from './services/log-in-out/logged-in.guard';
import { DataService } from './services/data/data.service';

// Third Party Services
import { CalendarModule } from 'primeng/calendar';
import { TreeModule } from 'primeng/tree';
import { DropdownModule } from 'primeng/dropdown';
import { FileUploadModule } from 'primeng/fileupload';
import { AgGridModule } from 'ag-grid-angular'; 

// Guards
import { JhaUnsavedChangesGuard } from '@jha/jharesponsive/jha-unsaved-changes';

// Primary Site Components
import { AppComponent } from './app.component';
import { MainHeaderComponent } from './components/main-header/main-header.component';
import { MainNavComponent } from './components/main-nav/main-nav.component';
import { HomeComponent } from './components/home/home.component';
import { AboutComponent } from './components/about/about.component';

// Pattern Components
import { AnimationComponent } from './components/responsive-patterns/animation/animation.component';
import { BreadcrumbsComponent } from './components/responsive-patterns/breadcrumbs/breadcrumbs.component';
import { BusyIndicatorComponent } from './components/responsive-patterns/busy-indicator/busy-indicator.component';
import { ButtonsComponent } from './components/responsive-patterns/buttons/buttons.component';
import { CarouselComponent } from './components/responsive-patterns/carousel/carousel.component';
import { ChangePasswordComponent } from './components/responsive-patterns/change-password/change-password.component';
import { ChartsComponent } from './components/responsive-patterns/charts/charts.component';
import { ContentLayoutComponent } from './components/responsive-patterns/content-layout/content-layout.component';
import { ContextMenuComponent } from './components/responsive-patterns/context-menu/context-menu.component';
import { CrudComponent } from './components/responsive-patterns/crud/crud.component';
import { DashboardComponent } from './components/responsive-patterns/dashboard/dashboard.component';
import { DataDenseComponent } from './components/responsive-patterns/data-dense/data-dense.component';
import { DialogBoxComponent } from './components/responsive-patterns/dialog-boxes/dialog-boxes.component';
import { DisplayBlockComponent } from './components/responsive-patterns/display-block/display-block.component';
import { DisplayBlockCollapseComponent } from './components/responsive-patterns/display-block-collapse/display-block-collapse.component';
import { DragAndDropComponent } from './components/responsive-patterns/drag-and-drop/drag-and-drop.component';
import { EditingFormDataComponent } from './components/responsive-patterns/editing-form-data/editing-form-data.component';
import { EnterpriseIconsComponent } from './components/responsive-patterns/enterprise-icons/enterprise-icons.component';
import { FileUploadComponent } from './components/responsive-patterns/file-upload/file-upload.component';
import { FixedLayoutComponent } from './components/responsive-patterns/fixed-layout/fixed-layout.component';
import { FixedLayoutSplitterComponent } from './components/responsive-patterns/fixed-layout-splitter/fixed-layout-splitter.component';
import { GridComponent } from './components/responsive-patterns/grid/grid.component';
import { GridComplexComponent } from './components/responsive-patterns/grid-complex/grid-complex.component';
import { GridTreeComponent } from './components/responsive-patterns/grid-tree/grid-tree.component';
import { GroupBoxesComponent } from './components/responsive-patterns/group-boxes/group-boxes.component';
import { HeadshotPhotosComponent } from './components/responsive-patterns/headshot-photos/headshot-photos.component';
import { InactivityTimeoutComponent } from './components/responsive-patterns/inactivity-timeout/inactivity-timeout.component';
import { LargeDatasetsComponent } from './components/responsive-patterns/large-datasets/large-datasets.component';
import { LargeFormAccordionComponent } from './components/responsive-patterns/large-form-accordion/large-form-accordion.component';
import { LargeFormWizardComponent } from './components/responsive-patterns/large-form-wizard/large-form-wizard.component';
import { ListCategorizationComponent } from './components/responsive-patterns/list-categorization/list-categorization.component';
import { LogInComponent } from './components/responsive-patterns/log-in/log-in.component';
import { MasterDetailButtonsComponent } from './components/responsive-patterns/master-detail-buttons/master-detail-buttons.component';
import { MasterDetailGridComponent } from './components/responsive-patterns/master-detail-grid/master-detail-grid.component';
import { MasterDetailGridMultipleComponent } from './components/responsive-patterns/master-detail-grid-multiple/master-detail-grid-multiple.component';
import { MasterDetailTreeViewComponent } from './components/responsive-patterns/master-detail-tree-view/master-detail-tree-view.component';
import { NotificationsComponent } from './components/responsive-patterns/notifications/notifications.component';
import { PercentageCircleComponent } from './components/responsive-patterns/percentage-circle/percentage-circle.component';
import { PlugInComponent } from './components/responsive-patterns/plug-in/plug-in.component';
import { ProcessTimelineComponent } from './components/responsive-patterns/process-timeline/process-timeline.component';
import { ProgressBarComponent } from './components/responsive-patterns/progress-bar/progress-bar.component';
import { RatingComponent } from './components/responsive-patterns/rating/rating.component';
import { RecordDetailComponent } from './components/responsive-patterns/record-detail/record-detail.component';
import { ResponsiveTableComponent } from './components/responsive-patterns/responsive-table/responsive-table.component';
import { ResponsiveLayoutComponent } from './components/responsive-patterns/responsive-layout/responsive-layout.component';
import { SearchComponent } from './components/responsive-patterns/search/search.component';
import { SessionTimeoutComponent } from './components/responsive-patterns/session-timeout/session-timeout.component';
import { SliderComponent } from './components/responsive-patterns/slider/slider.component';
import { StatusIconComponent } from './components/responsive-patterns/status-icon/status-icon.component';
import { StackedViewsComponent } from './components/responsive-patterns/stacked-views/stacked-views.component';
import { SelectAddressComponent } from './components/responsive-patterns/stacked-views/components/select-address/select-address.component';
import { EditAddressComponent } from './components/responsive-patterns/stacked-views/components/edit-address/edit-address.component';
import { StepsComponent } from './components/responsive-patterns/steps/steps.component';
import { SubheaderComponent } from './components/responsive-patterns/subheader/subheader.component';
import { TabsComponent } from './components/responsive-patterns/tabs/tabs.component';
import { TagsComponent } from './components/responsive-patterns/tags/tags.component';
import { TemplatedListComponent } from './components/responsive-patterns/templated-list/templated-list.component';
import { TextComponent } from './components/responsive-patterns/text/text.component';
import { ThemedSvgImagesComponent } from './components/responsive-patterns/themed-svg-images/themed-svg-images.component';
import { ThemingComponent } from './components/responsive-patterns/theming/theming.component';
import { TilesComponent } from './components/responsive-patterns/tiles/tiles.component';
import { ToolbarComponent } from './components/responsive-patterns/toolbar/toolbar.component';
import { TooltipsComponent } from './components/responsive-patterns/tooltips/tooltips.component';
import { TreeViewComponent } from './components/responsive-patterns/tree-view/tree-view.component';
import { UnsavedChangesComponent } from './components/responsive-patterns/unsaved-changes/unsaved-changes.component';
import { WizardComponent } from './components/responsive-patterns/wizard/wizard.component';

enableProdMode();

const routes: Routes =
	[
		{ path: '', redirectTo: '/Home', pathMatch: 'full' },
		{ path: 'Home', component: HomeComponent },
		{ path: 'About', component: AboutComponent },

		{ path: 'Animation', component: AnimationComponent },
		{ path: 'Breadcrumbs', component: BreadcrumbsComponent },
		{ path: 'BusyIndicator', component: BusyIndicatorComponent },
		{ path: 'Buttons', component: ButtonsComponent },
		{ path: 'Carousel', component: CarouselComponent },
		{ path: 'ChangePassword', component: ChangePasswordComponent },
		{ path: 'Charts', component: ChartsComponent },
		{ path: 'Crud', component: CrudComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
		{ path: 'ContentLayout', component: ContentLayoutComponent },
		{ path: 'ContextMenu', component: ContextMenuComponent },
		{ path: 'Dashboard', component: DashboardComponent },
		{ path: 'DataDense', component: DataDenseComponent },
		{ path: 'DialogBox', component: DialogBoxComponent },
		{ path: 'DisplayBlock', component: DisplayBlockComponent },
		{ path: 'DisplayBlockCollapse', component: DisplayBlockCollapseComponent },
		{ path: 'DragAndDrop', component: DragAndDropComponent },
		{ path: 'EditingFormData', component: EditingFormDataComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
		{ path: 'EnterpriseIcons', component: EnterpriseIconsComponent },
		{ path: 'FileUpload', component: FileUploadComponent },
		{ path: 'FixedLayout', component: FixedLayoutComponent },
		{ path: 'FixedLayoutSplitter', component: FixedLayoutSplitterComponent },
		{ path: 'Grid', component: GridComponent },
		{ path: 'GridComplex', component: GridComplexComponent },
		{ path: 'GridTree', component: GridTreeComponent },
		{ path: 'GroupBox', component: GroupBoxesComponent },
		{ path: 'HeadshotPhotos', component: HeadshotPhotosComponent },
		{ path: 'InactivityTimeout', component: InactivityTimeoutComponent },
		{ path: 'LargeDatasets', component: LargeDatasetsComponent },
		{ path: 'LargeFormsWizard', component: LargeFormWizardComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
		{ path: 'LargeFormsAccordion', component: LargeFormAccordionComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
		{ path: 'LazyLoad', loadChildren: () => import('./modules/lazy-load/lazy-load.module').then(m => m.LazyLoadModule) },
		{ path: 'ListCategorization', component: ListCategorizationComponent },
		{ path: 'LogIn', component: LogInComponent },
		{ path: 'MasterDetailButtons', component: MasterDetailButtonsComponent },
		{ path: 'MasterDetailGrid', component: MasterDetailGridComponent },
		{ path: 'MasterDetailGridMultiple', component: MasterDetailGridMultipleComponent },
		{ path: 'MasterDetailTreeView', component: MasterDetailTreeViewComponent },
		{ path: 'Notifications', component: NotificationsComponent },
		{ path: 'PercentageCircle', component: PercentageCircleComponent },
		{ path: 'PlugIn', component: PlugInComponent },
		{ path: 'ProcessTimeline', component: ProcessTimelineComponent },
		{ path: 'ProgressBar', component: ProgressBarComponent },
		{ path: 'Rating', component: RatingComponent },
		{ path: 'RecordDetail', component: RecordDetailComponent },
		{ path: 'ResponsiveTable', component: ResponsiveTableComponent },
		{ path: 'ResponsiveLayout', component: ResponsiveLayoutComponent },
		{ path: 'Search', component: SearchComponent },
		{ path: 'SearchPhone', component: SearchComponent },
		{ path: 'SessionTimeout', component: SessionTimeoutComponent },
		{ path: 'Slider', component: SliderComponent },
		{ path: 'StackedViews', component: StackedViewsComponent, canDeactivate: [JhaUnsavedChangesGuard] },
		{ path: 'StatusIcon', component: StatusIconComponent },
		{ path: 'Steps', component: StepsComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
		{ path: 'Subheader', component: SubheaderComponent },
		{ path: 'Tabs', component: TabsComponent },
		{ path: 'Tags', component: TagsComponent },
		{ path: 'TemplatedLists', component: TemplatedListComponent },
		{ path: 'Text', component: TextComponent },
		{ path: 'ThemedSvgImages', component: ThemedSvgImagesComponent },
		{ path: 'Theming', component: ThemingComponent },
		{ path: 'Tiles', component: TilesComponent },
		{ path: 'Toolbar', component: ToolbarComponent },
		{ path: 'Tooltips', component: TooltipsComponent },
		{ path: 'TreeView', component: TreeViewComponent },
		{ path: 'UnsavedChanges', component: UnsavedChangesComponent, canDeactivate: [JhaUnsavedChangesGuard] },
		{ path: 'Wizard', component: WizardComponent, canDeactivate: [JhaUnsavedChangesGuard]  },
		{ path: '**', component: HomeComponent }
	];

@NgModule({
	declarations: [
		AppComponent,
		MainHeaderComponent,
		MainNavComponent,
		HomeComponent,
		AboutComponent,
		AnimationComponent,
		BreadcrumbsComponent,
		BusyIndicatorComponent,
		ButtonsComponent,
		CarouselComponent,
		ChangePasswordComponent,
		ChartsComponent,
		ContentLayoutComponent,
		ContextMenuComponent,
		CrudComponent,
		DashboardComponent,
		DataDenseComponent,
		DialogBoxComponent,
		DisplayBlockComponent,
		DisplayBlockCollapseComponent,
		DragAndDropComponent,
		EditingFormDataComponent,
		EnterpriseIconsComponent,
		FileUploadComponent,
		FixedLayoutComponent,
		FixedLayoutSplitterComponent,
		GridComponent,
		GridComplexComponent,
		GridTreeComponent,
		GroupBoxesComponent,
		HeadshotPhotosComponent,
		InactivityTimeoutComponent,
		LargeDatasetsComponent,
		LargeFormAccordionComponent,
		LargeFormWizardComponent,
		ListCategorizationComponent,
		LogInComponent,
		MasterDetailButtonsComponent,
		MasterDetailGridComponent,
		MasterDetailGridMultipleComponent,
		MasterDetailTreeViewComponent,
		NotificationsComponent,
		PercentageCircleComponent,
		PlugInComponent,
		ProcessTimelineComponent,
		ProgressBarComponent,
		RatingComponent,
		RecordDetailComponent,
		ResponsiveTableComponent,
		ResponsiveLayoutComponent,
		SearchComponent,
		SessionTimeoutComponent,
		SliderComponent,
		StackedViewsComponent,
		SelectAddressComponent,
		EditAddressComponent,
		StatusIconComponent,
		StepsComponent,
		SubheaderComponent,
		TabsComponent,
		TagsComponent,
		TemplatedListComponent,
		TextComponent,
		ThemedSvgImagesComponent,
		ThemingComponent,
		TilesComponent,
		ToolbarComponent,
		TooltipsComponent,
		TreeViewComponent,
		UnsavedChangesComponent,
		WizardComponent
	],
	imports: [
		BrowserModule,
		BrowserAnimationsModule,
		FormsModule,
		ReactiveFormsModule,
		HttpClientModule,
		JhaButtonsModule,
		JhaDialogBoxModule,
		JhaFloatingToolbarModule,
		JhaFormsModule.forRoot(),
		JhaHeaderModule,
		JhaIconModule,
		JhaLayoutModule,
		JhaMasterDetailModule,
		JhaMegaMenuModule,
		JhaNavModule,
		JhaNotificationsModule.forRoot(),
		JhaPasswordStrengthMeterModule,
		JhaPercentageCircleModule,
		JhaPipesModule,
		JhaProcessTimelineModule,
		JhaProgressBarModule,
		JhaRatingModule,
		JhaRecordDetailModule,
		JhaResponsiveCoreModule.forRoot(),
		JhaTileModule,
		JhaToggleModule,
		JhaUnsavedChangesModule.forRoot(),
		RouterModule.forRoot(routes, {}),
		ButtonsModule.forRoot(),
		PaginationModule.forRoot(),
		TypeaheadModule.forRoot(),
		ContextMenuModule,
		CheckboxModule,
		MultiSelectModule,
		CalendarModule,
		RadioButtonModule,
		TreeModule,
		DropdownModule,
		FileUploadModule,
		StepsModule,
		TableModule,
		TreeTableModule,
		InputNumberModule,
		InputMaskModule,
		PickListModule,
		MatLegacyChipsModule,
		MatLegacySelectModule,
		MatLegacyTabsModule,
		AgGridModule,
		AngularSplitModule
	],
	providers: [
		{ provide: LocationStrategy, useClass: HashLocationStrategy },
		LogInOutService,
		LoggedInGuard,
		DataService
	],
	bootstrap: [AppComponent]
})
export class AppModule { }
