import { Component, OnInit, OnDestroy, HostListener } from '@angular/core';
import { Router } from '@angular/router';
import { JhaConstants } from '@jha/jharesponsive/jha-responsive-core';
import { JhaPubSubService, JhaPubSubMessage  } from '@jha/jharesponsive/jha-responsive-core';
import { LogInOutService } from './../../services/log-in-out/log-in-out.service';

import { JhaThemeColor, JhaThemingService } from '@jha/jharesponsive/jha-theme';

@Component({
	selector: 'main-header',
	templateUrl: './main-header.component.html',
	styleUrls: ['./main-header.component.scss'],
	providers: [JhaThemingService]
})
export class MainHeaderComponent implements OnInit, OnDestroy {
	public isMaximized: boolean = false;
	public hideHeader: boolean = false;
	public displayBackButton: boolean = false;
	public displayHeaderSearch: boolean = true;
	public loggedIn: boolean = false;
	private subscription: any = null;
	public version: string = JhaConstants.ControlsVersion;
	public themeColors: JhaThemeColor[];
	public showLogOutQuestion: boolean = false;
	public stateList: string[] =
		[
			'Alabama',
			'Alaska',
			'Arizona',
			'Arkansas',
			'California',
			'Colorado',
			'Connecticut',
			'Delaware',
			'Florida',
			'Georgia',
			'Hawaii',
			'Idaho',
			'Illinois',
			'Indiana',
			'Iowa',
			'Kansas',
			'Kentucky',
			'Louisiana',
			'Maine',
			'Maryland',
			'Massachusetts',
			'Michigan',
			'Minnesota',
			'Mississippi',
			'Missouri',
			'Montana',
			'Nebraska',
			'Nevada',
			'New Hampshire',
			'New Jersey',
			'New Mexico',
			'New York',
			'North Carolina',
			'North Dakota',
			'Ohio',
			'Oklahoma',
			'Oregon',
			'Pennsylvania',
			'Rhode Island',
			'South Carolina',
			'South Dakota',
			'Tennessee',
			'Texas',
			'Utah',
			'Vermont',
			'Virginia',
			'Washington',
			'West Virginia',
			'Wisconsin',
			'Wyoming'
		];
	private resizeTimeout: any;

	// Start/stop vertical scrolling mode if using fixed layout
	@HostListener('window:resize', ['$event'])
	onResize(event: Event): void {
		// Using a timer so we don't run this processing on every single fire of this event
		if (this.resizeTimeout) {
			clearTimeout(this.resizeTimeout);
		}
		this.resizeTimeout = setTimeout((() => {
			this.checkWidth();
		}).bind(this), 250);
	}

	constructor(
		private router: Router,
		private pubSubService: JhaPubSubService,
		private logInOutService: LogInOutService,
		private themingService: JhaThemingService
	) {
		this.themeColors = themingService.ThemeColors;

		// Handle maximized/unmaximized events from preload.js
		window.addEventListener('JhaWindowMaximized', () => { this.isMaximized = true; }, false);
		window.addEventListener('JhaWindowUnmaximized', () => { this.isMaximized = false; }, false);
	}

	public ngOnInit(): void {
		this.checkWidth();

		this.subscription = this.pubSubService.Stream.subscribe(pubSubMessage => this.processMessage(pubSubMessage));
	}

	public ngOnDestroy(): void {
		this.subscription.unsubscribe();
	}

	public checkWidth(): void {
		this.displayHeaderSearch = window.innerWidth > 1024;
	}

	public processMessage(pubSubMessage: JhaPubSubMessage): void {
		if (pubSubMessage.message === 'LOGGEDIN') {
			this.loggedIn = true;
		} else if (pubSubMessage.message === 'LOGGEDOUT') {
			this.loggedIn = false;
		}
	}

	public changeThemeColor(themeIndex: number): void {
		if (themeIndex >= 0 && themeIndex < this.themeColors.length) {
			this.themingService.ThemePath = './assets/themes';
			this.themingService.LoadThemeColor(this.themeColors[themeIndex].name);
		}
	}

	public backButtonClicked(): void {
		window.history.back();
	}

	public search(searchString: string): void {
		this.router.navigate(['Search', { search: searchString }]);
	}

	public print(): void {
		window.print();
	}

}
