import { Component, OnInit, OnDestroy } from '@angular/core';
import { JhaThemeColor, JhaThemingService } from '@jha/jharesponsive/jha-theme';
import { JhaPubSubService, JhaPubSubMessage  } from '@jha/jharesponsive/jha-responsive-core';

@Component({
	selector: 'main-nav',
	templateUrl: './main-nav.component.html',
	styleUrls: ['./main-nav.component.scss'],
	providers: [JhaThemingService]
})
export class MainNavComponent implements OnInit, OnDestroy {
	public hideNav: boolean = false;
	public loggedIn: boolean = false;
	private subscription: any = null;
	public themeColors: JhaThemeColor[];
	public stateList: string[] =
	[
		'Alabama',
		'Alaska',
		'Arizona',
		'Arkansas',
		'California',
		'Colorado',
		'Connecticut',
		'Delaware',
		'Florida',
		'Georgia',
		'Hawaii',
		'Idaho',
		'Illinois',
		'Indiana',
		'Iowa',
		'Kansas',
		'Kentucky',
		'Louisiana',
		'Maine',
		'Maryland',
		'Massachusetts',
		'Michigan',
		'Minnesota',
		'Mississippi',
		'Missouri',
		'Montana',
		'Nebraska',
		'Nevada',
		'New Hampshire',
		'New Jersey',
		'New Mexico',
		'New York',
		'North Carolina',
		'North Dakota',
		'Ohio',
		'Oklahoma',
		'Oregon',
		'Pennsylvania',
		'Rhode Island',
		'South Carolina',
		'South Dakota',
		'Tennessee',
		'Texas',
		'Utah',
		'Vermont',
		'Virginia',
		'Washington',
		'West Virginia',
		'Wisconsin',
		'Wyoming'
	];
	public userList: any[] =
	[
		{ name: 'Dorothy Bridges', street: '27 Ketch Harbour St.', cityStateZip: 'Stow, OH 44224'},
		{ name: 'Jerome Lee', street: '205 Bradford Ave.', cityStateZip: 'Attleboro, MA 02703'},
		{ name: 'Angel Ramos', street: '7859 W. Durham Lane', cityStateZip: 'Ballston Spa, NY 12020'}
	];

	constructor(private pubSubService: JhaPubSubService, private themingService: JhaThemingService) { }

	public ngOnInit(): void {
		this.themeColors = this.themingService.ThemeColors;
		this.subscription = this.pubSubService.Stream.subscribe(pubSubMessage => this.processMessage(pubSubMessage));
	}

	public ngOnDestroy(): void {
		this.subscription.unsubscribe();
	}

	public processMessage(pubSubMessage: JhaPubSubMessage): void {
		if (pubSubMessage.message === 'LOGGEDIN') {
			this.loggedIn = true;
		} else if (pubSubMessage.message === 'LOGGEDOUT') {
			this.loggedIn = false;
		}
	}

	public changeThemeColor(themeIndex: number): void {
		if (themeIndex >= 0 && themeIndex < this.themeColors.length) {
			// Specify where the relative location for our theme files
			this.themingService.ThemePath = './assets/themes';

			// Load the new theme.
			this.themingService.LoadThemeColor(this.themeColors[themeIndex].name);

			// Let other components know that theme has changed
			this.pubSubService.Stream.next(new JhaPubSubMessage('THEME_CHANGED', null));
		}
	}
}
