import { Component, OnInit, ViewChild } from '@angular/core';
import { Tickler } from '../../../classes/Tickler';
import { ServiceEventNearPastDate } from '../../../classes/ServiceEventNearPastDate';
import { JhaThemingService } from '@jha/jharesponsive/jha-theme';
import { JhaPubSubService, JhaPubSubMessage } from '@jha/jharesponsive/jha-responsive-core';

import Chart from 'chart.js/auto';

@Component({
	selector: 'data-dense',
	templateUrl: './data-dense.component.html',
	styleUrls: ['./data-dense.component.scss'],
	providers: [JhaThemingService]
})
export class DataDenseComponent implements OnInit {
	public loading: boolean = false;
	public fromTicklerDate: Date = new Date('3/21/2017');
	public toTicklerDate: Date = new Date('4/18/2017');
	public appointmentDate: Date = new Date();
	public filterByDate: boolean = true;
	public filterInfo: string = '';
	public eventsNearPastDate: ServiceEventNearPastDate[];
	public sortEventsColumn: string;
	public sortEventsDirection: number = 1;
	public ticklers: Tickler[];
	public sortTicklersColumn: string;
	public sortTicklersDirection: number = 1;
    private durationChart: any = null;
    private animateCharts: Boolean = true;
    private chartFontColor: String = '';
    private chartGridColor: String = '';
    private chartBackgroundColor1: String = '';
    private chartBackgroundColor2: String = '';
	private subscription: any = null;

	@ViewChild('calendar', { static: false })
	calendar: any;

	constructor(private themingService: JhaThemingService, private pubSubService: JhaPubSubService) {
		this.subscription = this.pubSubService.Stream.subscribe(pubSubMessage => this.processMessage(pubSubMessage));
	 }

	ngOnInit(): void {
		// FAKE UP DATA
		this.eventsNearPastDate =
			[
				new ServiceEventNearPastDate('Near', 'Elizabeth', 'Powell', 'Return call', new Date()),
				new ServiceEventNearPastDate('Near', 'Keith', 'Johnson', 'Status Report', new Date())
			];
		this.sortEventsNearPast('NeededBy');

		this.ticklers =
			[
				new Tickler(new Date('6/11/2016'), 'Tamera Lynn Dowling', 'New Accounts'),
				new Tickler(new Date('3/2/2016'), 'Huda Mohammed Al-sultan', 'Account Test'),
				new Tickler(new Date('3/1/2016'), 'Wanda L Reno', 'First Phone Contact'),
				new Tickler(new Date('3/1/2016'), 'Samuel Adams', 'Client Contact'),
				new Tickler(new Date('3/1/2016'), 'Qiong Wang', 'Consolidation')
			];
		this.sortTicklers('NeededBy');

		this.calculateFilterInfo();
		
		// Check if the user requests no animation
        const mediaQuery = window.matchMedia("(prefers-reduced-motion: reduce)");
        if (!mediaQuery || mediaQuery.matches) {
            this.animateCharts = false;
        }

		this.InitializeChart();
	}

	public processMessage(pubSubMessage: JhaPubSubMessage): void {
		if (pubSubMessage.message === 'THEME_CHANGED') {
			this.InitializeChart();
		}
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

	public sortEventsNearPast(sortColumn: string): any {
		// Reverse the sort direction
		this.sortEventsDirection *= -1;
		const sortDirection: number = this.sortEventsDirection;

		// Remember the sort column
		this.sortEventsColumn = sortColumn;

		this.eventsNearPastDate.sort((itemA: ServiceEventNearPastDate, itemB: ServiceEventNearPastDate) => {
			// Compare ascendening
			let sortResult = 0;
			if (itemA[sortColumn] < itemB[sortColumn]) {
				sortResult = -1;
			} else if (itemA[sortColumn] > itemB[sortColumn]) {
				sortResult = 1;
			}

			// Adjust for sort direction
			sortResult *= sortDirection;

			return sortResult;
		});
	}

	public sortTicklers(sortColumn: string): any {
		// Reverse the sort direction
		this.sortTicklersDirection *= -1;
		const sortDirection: number = this.sortTicklersDirection;

		// Remember the sort column
		this.sortTicklersColumn = sortColumn;

		this.ticklers.sort((itemA: Tickler, itemB: Tickler) => {
			// Compare ascendening
			let sortResult = 0;
			if (itemA[sortColumn] < itemB[sortColumn]) {
				sortResult = -1;
			} else if (itemA[sortColumn] > itemB[sortColumn]) {
				sortResult = 1;
			}

			// Adjust for sort direction
			sortResult *= sortDirection;

			return sortResult;
		});
	}

	public subtractApptDate(): void {
		this.appointmentDate = new Date(this.appointmentDate.setDate(this.appointmentDate.getDate() - 1));
	}

	public addApptDate(): void {
		this.appointmentDate = new Date(this.appointmentDate.setDate(this.appointmentDate.getDate() + 1));
	}

	public updateTicklersFilters(): void {
		this.calculateFilterInfo();
	}

	private calculateFilterInfo(): void {
		let filterInfo: string = 'None';
		if (this.filterByDate) {
			filterInfo = this.getDateString(this.fromTicklerDate) + ' to ' + this.getDateString(this.toTicklerDate);
		}
		this.filterInfo = 'Filters: ' + filterInfo;
	}

	private getDateString(dateVal: Date): any {
		const monthVal = dateVal.getMonth() + 1;
		const dayVal = dateVal.getDate();
		const monthStr = monthVal > 9 ? monthVal : '0' + monthVal;
		const dayStr = dayVal > 9 ? dayVal : '0' + dayVal;

		return monthStr + '/' + dayStr + '/' + dateVal.getFullYear();
	}

	// Initialize Chart
	InitializeChart(): void {
		setTimeout(() => {
			this.setUpDurationChart();
		}, 100);
	}

	// Build or update chart
    private setUpDurationChart() {
        const canvas = document.querySelector('#durationChart');
		this.chartFontColor = this.themingService.GetChartFontColor();
		this.chartGridColor = this.themingService.GetChartGridColor();
		this.chartBackgroundColor1 = String(this.themingService.GetChartDataSeriesColor(1));
		this.chartBackgroundColor2 = this.themingService.GetChartDataSeriesColor(2);
		
        if (canvas) {
            if (this.durationChart) {
                // If chart already exists, update it with current theme's chart colors
                this.durationChart.data.datasets[0].backgroundColor = this.chartBackgroundColor1;
                this.durationChart.data.datasets[1].backgroundColor = this.chartBackgroundColor2;
                this.durationChart.options.plugins.legend.labels.color = this.chartFontColor;
                this.durationChart.options.scales.x.ticks.color = this.chartFontColor;
                this.durationChart.options.scales.x.grid.color = this.chartGridColor;
                this.durationChart.options.scales.y.ticks.color = this.chartFontColor;
                this.durationChart.options.scales.y.grid.color = this.chartGridColor;
                this.durationChart.update();
            } else {
                // Otherwise create a new chart using current theme's chart colors
                this.durationChart = new Chart(canvas as any, {
                    type: 'bar',
                    data: {
                        labels: ['January', 'February', 'March', 'April', 'May', 'June', 'July'],
                        datasets: [
                            {
                                label: 'Primary',
                                backgroundColor: this.chartBackgroundColor1 as any,
                                data: [65, 59, 80, 81, 56, 55, 40]
                            },
                            {
                                label: 'Secondary',
                                backgroundColor: this.chartBackgroundColor2,
                                data: [28, 48, 40, 19, 86, 27, 90]
                            }
                        ]
                    },
                    options: {
                        ...(!this.animateCharts && { animation: false }),
                        responsive: true,
                        maintainAspectRatio: false,
                        plugins: {
                            legend: {
                                labels: {
                                    color: this.chartFontColor as any
                                }
                            }
                        },
                        scales:
                        {
                            x: {
                                grid: { color: this.chartGridColor as any },
                                ticks: { color: this.chartFontColor as any }
                            },
                            y: {
                                grid: { color: this.chartGridColor as any },
                                ticks: { color: this.chartFontColor as any }
                            }
                        }
                    }
                });
            }
        }
    }

	openCalendar(event: Event): void {
		this.calendar.showOverlay(this.calendar.inputfieldViewChild.nativeElement);
		this.appointmentDate = new Date(this.appointmentDate.setDate(this.appointmentDate.getDate()));
		event.stopPropagation();
	}

}
