import { Component } from '@angular/core';
import { UntypedFormGroup, UntypedFormControl, Validators } from '@angular/forms';


@Component({
	selector: 'dialog-boxes',
	templateUrl: './dialog-boxes.component.html',
	styleUrls: ['./dialog-boxes.component.scss']
})
export class DialogBoxComponent {
	public showQuestion: boolean = false;
	public showQuestionWithDestructiveOption: boolean = false;
	public showDialog: boolean = false;
	public showBeforeCloseDialog: boolean = false;
	public canDialogClose: boolean = true;

	public firstNameVal: string = '';
	public lastNameVal: string = '';

	public addUserForm: UntypedFormGroup = new UntypedFormGroup({
		'firstName': new UntypedFormControl(this.firstNameVal, Validators.required),
		'lastName': new UntypedFormControl(this.lastNameVal, Validators.required)
	});

	get firstName(): any { return this.addUserForm.get('firstName'); }
	get lastName(): any { return this.addUserForm.get('lastName'); }

	public dialogBeforeClosed(dialogResult: string): void {
		console.log('canDialogClose is ' + this.canDialogClose);
		console.log('button clicked: ' + dialogResult);

		// Simulate delay for testing whether dialog should close or not
		setTimeout(() => {
			this.canDialogClose = true;
			console.log('canDialogClose has been changed to ' + this.canDialogClose);
		}, 400);
	}

	public dialogClosed(dialogResult: string): void {
		if (dialogResult === 'Save') {
			// We get a dialogResult of "OK" when the user presses the OK button
			alert('Process form changes');
		} else if (dialogResult === 'Cancel' || dialogResult === 'Escape') {
			// We get a dialogResult of "Cancel" when the user presses the Cancel button
			// and a dialogResult of "Escape" when the user presses the Escape key.
			// We map the Escape key case to the least destructive path through the dialog.
			alert('Cancel form changes');
		}
	}

	public questionClosed(dialogResult: string): void {
		alert('You answered ' + dialogResult);
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}
}
