import { Component } from '@angular/core';
import { DataService } from '../../../services/data/data.service';
import { JhaNotificationService, JhaNotificationTypes } from '@jha/jharesponsive/jha-notifications';

@Component({
	selector: 'file-upload',
	templateUrl: './file-upload.component.html',
	styleUrls: ['./file-upload.component.scss']
})
export class FileUploadComponent {
	public uploadedFiles: any[] = [];

	constructor(
		private dataService: DataService,
		private jhaNotificationService: JhaNotificationService
	) { }

	public onStandardUpload(event: any, standardFileUpload: any): void {
		// Keep track of all files uploaded; these are displayed in the control
		for (const file of event.files) {
			this.uploadedFiles.push(file);
		}

		// Clear out the file upload control
		standardFileUpload.clear();

		// Display a popup notification
		this.jhaNotificationService.DisplayPopupNotification('File upload complete', JhaNotificationTypes.Success);
	}

	public onBasicUpload(event: any, basicFileUpload: any): void {
		// Clear out the basic file upload control
		basicFileUpload.clear();

		// Manually upload the file(s)
		this.dataService.postFile(this.uploadedFiles).subscribe(data => {
			// Display a popup notification on success
			this.jhaNotificationService.DisplayPopupNotification('File upload complete', JhaNotificationTypes.Success);
		}, error => {
			// Handle error
		  	console.log(error);
		});
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}
}
