import { Component, OnInit } from '@angular/core';
import 'ag-grid-enterprise';
import { takeWhile } from 'rxjs/operators';
import { JhaButtonGroupOption } from '@jha/jharesponsive/jha-buttons';
import { UserSearchService } from '../../../services/users/users.service';

@Component({
	selector: 'grid-complex',
	templateUrl: './grid-complex.component.html',
	styleUrls: ['./grid-complex.component.scss'],
	providers: [UserSearchService]
})
export class GridComplexComponent implements OnInit {
	private gridApi;
	private gridColumnApi;
	public columnDefs;
	public groupedColumnDefs;
	public autoGroupColumnDef;
	public defaultColDef;
	private columnTypes;
	private alive: boolean = true;
	public rowData;
	public loadError: string = '';
	public stylingOptions: JhaButtonGroupOption[] = [
		new JhaButtonGroupOption('Minimal', 'Minimal'),
		new JhaButtonGroupOption('Striped', 'Striped')
	];
	public stylingOption: string = 'Minimal';

	constructor(private userService: UserSearchService) {

		this.defaultColDef = {
			width: 150,
			editable: true,
			filter: 'agTextColumnFilter',
			sortable: true,
			resizable: true
		};

		this.columnDefs = [
			{
				headerName: 'Contact Information', children: [
					{ headerName: 'Name', field: 'Name', checkboxSelection: true, floatingFilter: true },
					{ headerName: 'Phone', field: 'Phone', floatingFilter: true },
					{ headerName: 'Email', field: 'Email', floatingFilter: true }
				]
			},
			{
				headerName: 'Location Information', children: [
					{ headerName: 'Address', field: 'Address', floatingFilter: true },
					{ headerName: 'City', field: 'City', floatingFilter: true },
					{ headerName: 'Region', field: 'Region', floatingFilter: true },
					{ headerName: 'Country', field: 'Country', floatingFilter: true },
					{ headerName: 'Zip', field: 'Zip', floatingFilter: true }
				]
			},
			{ headerName: 'Company', field: 'Company', sortable: false, floatingFilter: true }
		];

		this.groupedColumnDefs = [
			{ headerName: 'Company', field: 'Company', rowGroup: true, hide: true, floatingFilter: true },
			{ headerName: 'Name', field: 'Name', floatingFilter: true },
			{ headerName: 'Phone', field: 'Phone', floatingFilter: true },
			{ headerName: 'Email', field: 'Email', floatingFilter: true },
			{ headerName: 'Address', field: 'Address', floatingFilter: true },
			{ headerName: 'City', field: 'City', floatingFilter: true },
			{ headerName: 'Region', field: 'Region', floatingFilter: true },
			{ headerName: 'Country', field: 'Country', floatingFilter: true },
			{ headerName: 'Zip', field: 'Zip', floatingFilter: true }
		];

		this.autoGroupColumnDef = { minWidth: 200 };
	}

	ngOnInit(): void {
		this.startUserSearch();
	}

	private startUserSearch(): any {
		this.userService.GetAllUsers().pipe(takeWhile(() => this.alive)).subscribe(
			data => {
				this.rowData = data;
			},
			error => this.loadError = <any>error
		);
	}

	public onGridReady(params): void {
		this.gridApi = params.api;
		this.gridColumnApi = params.columnApi;

		params.api.sizeColumnsToFit();
		window.addEventListener('resize', function()  {
			setTimeout(function() {
				params.api.sizeColumnsToFit();
			});
		});
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
