import { Component, OnInit, ViewChild } from '@angular/core';
import { Router } from '@angular/router';
import { JhaButtonGroupOption } from '@jha/jharesponsive/jha-buttons';
import { DataService } from '../../../services/data/data.service';
import { Table } from 'primeng/table';
import { TreeNode, SelectItem } from 'primeng/api';

@Component({
	// tslint:disable-next-line:component-selector
	selector: 'grid-tree',
	templateUrl: './grid-tree.component.html',
	styleUrls: ['./grid-tree.component.scss']
})
export class GridTreeComponent implements OnInit {
	public files: TreeNode[];
	public cols: any[];
	public selectedNode1: TreeNode;
	public fileTypeList: SelectItem[];
	public stylingOptions: JhaButtonGroupOption[] = [
		new JhaButtonGroupOption('Minimal', 'Minimal'),
		new JhaButtonGroupOption('Striped', 'Striped')
	];
	public stylingOption: string = 'Minimal';

	@ViewChild('tt') table: Table;

	constructor(private dataService: DataService, private router: Router) { }

	ngOnInit(): void {
		// Get list of files
		this.dataService.getFileSystem().then((files) => { this.files = files; });

		// Get list of columns
		this.dataService.getFileSystemCols().then((cols) => { this.cols = cols; });

		// Get list of file types
		this.dataService.getFileTypeList().then((fileTypeList) => { this.fileTypeList = fileTypeList; });
	}

	public onFileTypeChange(event): void {
		this.table.filter(event.value, 'type', 'in');
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
