import { Component, OnInit } from '@angular/core';
import { UntypedFormGroup, UntypedFormControl, Validators, ValidationErrors } from '@angular/forms';
import { SelectItem } from 'primeng/api';
import { MatChipInputEvent } from '@angular/material/chips';
import { COMMA, ENTER } from '@angular/cdk/keycodes';
import { lastValueFrom } from 'rxjs';

import { DataService } from '../../../services/data/data.service';
import { JhaUnsavedChangesService } from '@jha/jharesponsive/jha-unsaved-changes';

import { CountryAddressTypeValidator } from './country-address-type-validator.directive';

import { AlertGroup } from '../../../classes/alert-group';
import { CustomerDetails } from '../../../classes/customer-details';
import { JhaButtonGroupOption } from '@jha/jharesponsive/jha-buttons';
import { JhaCustomValidationMap } from '@jha/jharesponsive/jha-forms';

@Component({
	selector: 'large-form-accordion',
	templateUrl: './large-form-accordion.component.html',
	styleUrls: ['./large-form-accordion.component.scss']
})
export class LargeFormAccordionComponent implements OnInit {
	public loadingCustomerInfo: boolean = true;
	public postingCustomerInfo: boolean = false;
	public customerInfo: CustomerDetails =
		new CustomerDetails('', false, 0, 0, new Date(), 0, 0, false, [], '', '', null, '', '', '', 0, '', '', 'Domestic', '', '', '', '', null, '', null, null);
	public accountTypeList: SelectItem[] = [];
	public monthlyIncome: number = 0;
	public annualIncome: number = 0;
	public stateList: SelectItem[] = [];
	public countryList: string[] = [];
	public countyList: SelectItem[] = [];
	public alertGroups: AlertGroup[] = [];
	public showDialog: boolean = false;
	public richTextEditor: HTMLTextAreaElement;
	public recentAccounts: SelectItem[] =
		[
			{ label: 'John Doe', value: 'A000007' },
			{ label: 'Kevin Peterson', value: '141' },
			{ label: 'Frank Kristopoulos', value: 'A000013' },
			{ label: 'Fidel Martin', value: '80103' }
		];
	public selectedValues: string[] = ['Anteater', 'Bat', 'Cat', 'Dog'];
	public readonly separatorKeyCodes: number[] = [ENTER, COMMA];
	public animalList: string[] = ['Anteater', 'Bat', 'Cat', 'Dog', 'Elephant', 'Falcon', 'Gorilla', 'Hippo'];
	public typeaheadSelectionInProgess: boolean = false;
	public addressTypeOptions: JhaButtonGroupOption[] = [
		new JhaButtonGroupOption('Domestic', 'Domestic'),
		new JhaButtonGroupOption('International', 'International')
	];
	public selectedSectionId: string = 'CustomerInformation';
	public customValidationMaps: JhaCustomValidationMap[] = [
		new JhaCustomValidationMap('countryAddressTypeMismatch', ['country', 'addressType'])
	];
	public editForm: UntypedFormGroup = new UntypedFormGroup({
		'name': new UntypedFormControl(this.customerInfo.name, Validators.required),
		'ssn': new UntypedFormControl(this.customerInfo.ssn, Validators.required),
		'birthdate': new UntypedFormControl(this.customerInfo.birthdate, Validators.required),
		'lastLogin': new UntypedFormControl(this.customerInfo.lastLogin, Validators.required),
		'grossIncome': new UntypedFormControl(this.customerInfo.grossIncome),
		'interestRate': new UntypedFormControl(this.customerInfo.interestRate, [Validators.min(3), Validators.max(100)]),
		'accountTypes': new UntypedFormControl(this.customerInfo.accountTypes, Validators.required),
		'recentAccounts': new UntypedFormControl(this.recentAccounts, Validators.required),
		'activeCustomer': new UntypedFormControl(this.customerInfo.activeCustomer),
		'idChecked': new UntypedFormControl(this.customerInfo.idChecked),
		'street': new UntypedFormControl(this.customerInfo.street, Validators.required),
		'city': new UntypedFormControl(this.customerInfo.city, Validators.required),
		'state': new UntypedFormControl(this.customerInfo.state, Validators.required),
		'zip': new UntypedFormControl(this.customerInfo.zip, [Validators.required, Validators.pattern('^[0-9]{5}(?:[0-9]{4})?$')]),
		'county': new UntypedFormControl(this.customerInfo.county, Validators.required),
		'country': new UntypedFormControl(this.customerInfo.country, Validators.required),
		'addressType': new UntypedFormControl(this.customerInfo.addressType),
		'homePhone': new UntypedFormControl(this.customerInfo.homePhone),
		'businessPhone': new UntypedFormControl(this.customerInfo.businessPhone),
		'mobilePhone': new UntypedFormControl(this.customerInfo.mobilePhone),
		'email': new UntypedFormControl(this.customerInfo.email, Validators.email),
		'alerts': new UntypedFormControl(this.customerInfo.alerts, Validators.required),
		'comments': new UntypedFormControl(''),
		'editor': new UntypedFormControl(''),
		'selectedAnimal': new UntypedFormControl('')
	}, { validators: CountryAddressTypeValidator });

	get name(): any { return this.editForm.get('name'); }
	get ssn(): any { return this.editForm.get('ssn'); }
	get birthdate(): any { return this.editForm.get('birthdate'); }
	get lastLogin(): any { return this.editForm.get('lastLogin'); }
	get grossIncome(): any { return this.editForm.get('grossIncome'); }
	get interestRate(): any { return this.editForm.get('interestRate'); }
	get accountTypes(): any { return this.editForm.get('accountTypes'); }
	get activeCustomer(): any { return this.editForm.get('activeCustomer'); }
	get idChecked(): any { return this.editForm.get('idChecked'); }
	get street(): any { return this.editForm.get('street'); }
	get city(): any { return this.editForm.get('city'); }
	get state(): any { return this.editForm.get('state'); }
	get zip(): any { return this.editForm.get('zip'); }
	get county(): any { return this.editForm.get('county'); }
	get country(): any { return this.editForm.get('country'); }
	get addressType(): any { return this.editForm.get('addressType'); }
	get homePhone(): any { return this.editForm.get('homePhone'); }
	get businessPhone(): any { return this.editForm.get('businessPhone'); }
	get mobilePhone(): any { return this.editForm.get('mobilePhone'); }
	get email(): any { return this.editForm.get('email'); }
	get alerts(): any { return this.editForm.get('alerts'); }
	get comments(): any { return this.editForm.get('comments'); }
	get editor(): any { return this.editForm.get('editor'); }
	get selectedAnimal(): any { return this.editForm.get('selectedAnimal'); }

	constructor(
		private dataService: DataService,
		private unsavedChangesService: JhaUnsavedChangesService
	) {

		// Capture desktop and mobile dropdowns
		const stateDropDown: any = this.editForm.get('state');

		// Manually subscribe to value changes on state dropdown and propogate to all state form controls
		stateDropDown.valueChanges.subscribe((value: any) => stateDropDown.setValue(value, { emitEvent: false }));

		// Capture multiple account type inputs
		const accountTypeInputs: any = this.editForm.get('accountTypes');

		// Manually subscribe to value changes on account types and propogate to all account type form controls
		accountTypeInputs.valueChanges.subscribe((value: any) => accountTypeInputs.setValue(value, { emitEvent: false }));

	}

	ngOnInit(): void {
		// Load list of account types
		this.dataService.GetAccountTypeList().then((accountTypeList) => { this.accountTypeList = accountTypeList; });

		// Load list of US states
		this.dataService.GetStateList().then((stateList) => {
			this.stateList = [];
			stateList.forEach((stateOption) => { this.stateList.push({ 'label': stateOption.name, 'value': stateOption.value }); });
		});

		// Load list of counties
		this.dataService.GetCountyList().then((countyList) => { this.countyList = countyList; });

		// Load list of countries
		this.dataService.GetCountryList().then((countryList) => { this.countryList = countryList; });

		// Load list of alerts
		this.dataService.getAlertTypeList().then((alertGroups) => { this.alertGroups = alertGroups; });

		// Load customer info
		this.loadingCustomerInfo = true;
		this.dataService.getCustomerDetailInfo().then((customerDetailInfo) => {
			this.loadingCustomerInfo = false;
			this.customerInfo = customerDetailInfo;
			this.name.setValue(customerDetailInfo.name);
			this.ssn.setValue(customerDetailInfo.ssn);
			this.birthdate.setValue(customerDetailInfo.birthdate);
			this.lastLogin.setValue(customerDetailInfo.lastLogin);
			this.grossIncome.setValue(customerDetailInfo.grossIncome);
			this.interestRate.setValue(customerDetailInfo.interestRate);
			this.accountTypes.setValue(customerDetailInfo.accountTypes);
			this.activeCustomer.setValue(customerDetailInfo.activeCustomer);
			this.idChecked.setValue(customerDetailInfo.idChecked);
			this.street.setValue(customerDetailInfo.street);
			this.city.setValue(customerDetailInfo.city);
			this.state.setValue(customerDetailInfo.state);
			this.zip.setValue(customerDetailInfo.zip);
			this.county.setValue(customerDetailInfo.county);
			this.country.setValue(customerDetailInfo.country);
			this.addressType.setValue(customerDetailInfo.addressType);
			this.homePhone.setValue(customerDetailInfo.homePhone);
			this.businessPhone.setValue(customerDetailInfo.businessPhone);
			this.mobilePhone.setValue(customerDetailInfo.mobilePhone);
			this.email.setValue(customerDetailInfo.email);
			this.alerts.setValue(customerDetailInfo.alerts);
		});
	}

	// canDeactivate for unsaved changes guard
	public async canDeactivate() {
		if (this.editForm.dirty) {
			const promptResult =  this.unsavedChangesService.promptUnsavedChangesDialog(this.editForm.valid);
			
			let promptUserResult = await lastValueFrom(promptResult);
			
			if (promptUserResult === 'Save') {
				// Save changes in database, then return true
				console.log('Saving changes');
				alert('The user chose to save changes, so you should capture the changes and save them to a database here.');
				return true;
			} else if (promptUserResult === 'DontSave') {
				console.log('Dont save changes');
				return true;
			} else if (promptUserResult === 'LoseChanges') {
				console.log('Losing changes');
				return true;
			} else {
				console.log('Returning to editing without leaving page');
				return false;
			};
		} else {
			return true;
		}
	}

	// Save customer changes
	public saveChanges(): void {
		this.postingCustomerInfo = true;
		this.dataService.postCustomerDetailInfo(this.customerInfo).then((errorFlag) => {
			this.postingCustomerInfo = false;
			if (!errorFlag) {
				alert('All fields were successfully validated, so your app would typically navigate to a different view here');
				this.editForm.markAsPristine();
				this.editForm.markAsUntouched();
			}
		});
	}

	// Cancel customer changes
	public cancelChanges(): void {
		alert('This cancels the edit session, so your app would typically navigate to a different view here');
	}

	public DisplayMonthlyIncomeDialog(): void {
		this.annualIncome = this.grossIncome.value;
		this.monthlyIncome = this.annualIncome / 12;
		this.showDialog = true;
	}

	public updateAnnualIncome(): void {
		this.annualIncome = this.monthlyIncome * 12;
	}

	public closeDialog(dialogResult: string): void {
		if (dialogResult === 'Save') {
			this.grossIncome.setValue(this.annualIncome);
		}
	}

	public addValue(event: MatChipInputEvent): void {
		const input = event.input;
		const value = event.value;

		// Add our fruit
		if ((value || '').trim()) {
			this.selectedValues.push(value.trim());
		}

		// Reset the input value
		if (input) {
			input.value = '';
		}
	}

	public removeValue(valueToRemove: string): void {
		const index = this.selectedValues.indexOf(valueToRemove);
		if (index >= 0) {
			this.selectedValues.splice(index, 1);
		}
	}

	public addTag(event: MatChipInputEvent): void {
		// If typeahead is running, skip adding string
		setTimeout(() => {
			if (!this.typeaheadSelectionInProgess) {
				this.addSelectedValueString(event.value.trim());
			}
		}, 100);

		// Reset to false for next value submission
		this.typeaheadSelectionInProgess = false;
	}

	public onTypeaheadSelect(e: any): void {
		// Stop adding tag if typeahead option is available
		this.typeaheadSelectionInProgess = true;
		this.addSelectedValueString(e.item.trim());
	}

	public addSelectedValueString(value: string): void {
		// Push value to selected values array
		this.selectedValues.push(value);

		// Reset input value
		this.selectedAnimal.setValue('');
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}



	public introduceFakeErrors(): void {
		this.name.setValue('');
		this.name.markAsDirty();
		this.birthdate.setValue('');
		this.birthdate.markAsDirty();
		this.accountTypes.setValue('');
		this.accountTypes.markAsDirty();
		this.street.setValue('');
		this.street.markAsDirty();
		this.city.setValue('');
		this.city.markAsDirty();
		this.county.setValue('');
		this.county.markAsDirty();
	}

	public goToFormControl(formControlName: string): void {
		this.displaySectionForFieldId(formControlName);
		setTimeout(() => { this.focusField(formControlName); }, 100);
	}

	public focusField(formControlName: string): void {
		const nextControl = document.getElementById('field-' + formControlName);
		if (nextControl.getElementsByTagName('input')[0]) {
			// If element has child inputs, set focus on the first child input
			nextControl.getElementsByTagName('input')[0].focus();
		} else {
			// This is a standard input, just set focus
			nextControl.focus();
		}
	}

	public displaySectionForFieldId(formControlName: string): void {
		if (formControlName === 'name' ||
			formControlName === 'ssn' ||
			formControlName === 'birthdate' ||
			formControlName === 'lastLogin' ||
			formControlName === 'grossIncome' ||
			formControlName === 'interestRate' ||
			formControlName === 'accountTypes') {
			this.selectedSectionId = 'CustomerInformation';
		} else if (formControlName === 'street' ||
			formControlName === 'city' ||
			formControlName === 'state' ||
			formControlName === 'zip' ||
			formControlName === 'county' ||
			formControlName === 'country' ||
			formControlName === 'addressType') {
			this.selectedSectionId = 'Address';
		} else if (formControlName === 'homePhone' ||
			formControlName === 'businessPhone' ||
			formControlName === 'mobilePhone' ||
			formControlName === 'email' ||
			formControlName === 'alerts') {
			this.selectedSectionId = 'ContactInformation';
		} else if (formControlName === 'comments') {
			this.selectedSectionId = 'Comments';
		}
	}

	public customerInformationSectionHasError(): boolean {
		const hasError: boolean =
			this.nameFieldHasError() ||
			this.ssnFieldHasError() ||
			this.birthdateFieldHasError() ||
			this.lastLoginFieldHasError() ||
			this.grossIncomeFieldHasError() ||
			this.interestRateFieldHasError() ||
			this.accountTypesFieldHasError();

		return hasError;
	}

	public addressSectionHasError(): boolean {
		const hasError: boolean =
			this.streetFieldHasError() ||
			this.cityFieldHasError() ||
			this.stateFieldHasError() ||
			this.zipFieldHasError() ||
			this.countyFieldHasError() ||
			this.countryFieldHasError() ||
			this.addressTypeFieldHasError() ||
			this.countryAddressTypeMismatchHasError();

		return hasError;
	}

	public contactInformationSectionHasError(): boolean {
		const hasError: boolean =
			this.homePhoneFieldHasError() ||
			this.businessPhoneFieldHasError() ||
			this.mobilePhoneFieldHasError() ||
			this.emailFieldHasError() ||
			this.alertsFieldHasError();

		return hasError;
	}

	public commentsSectionHasError(): boolean {
		const hasError: boolean =
			this.commentsFieldHasError();

		return hasError;
	}

	private accountTypesFieldHasError(): boolean {
		return (this.accountTypes.invalid && (this.accountTypes.dirty || this.accountTypes.touched));
	}

	private interestRateFieldHasError(): boolean {
		return (this.interestRate.invalid && (this.interestRate.dirty || this.interestRate.touched));
	}

	private grossIncomeFieldHasError(): boolean {
		return (this.grossIncome.invalid && (this.grossIncome.dirty || this.grossIncome.touched));
	}

	private lastLoginFieldHasError(): boolean {
		return (this.lastLogin.invalid && (this.lastLogin.dirty || this.lastLogin.touched));
	}

	private birthdateFieldHasError(): boolean {
		return (this.birthdate.invalid && (this.birthdate.dirty || this.birthdate.touched));
	}

	private ssnFieldHasError(): boolean {
		return (this.ssn.invalid && (this.ssn.dirty || this.ssn.touched));
	}

	private nameFieldHasError(): boolean {
		return (this.name.invalid && (this.name.dirty || this.name.touched));
	}

	private addressTypeFieldHasError(): boolean {
		return (this.addressType.invalid && (this.addressType.dirty || this.addressType.touched));
	}

	private countryAddressTypeMismatchHasError(): boolean {
		let hasError: boolean = false;
		if (this.editForm.errors && (this.editForm.dirty || this.editForm.touched)) {
			Object.keys(this.editForm.errors).forEach(key => {
				const formErrors: ValidationErrors = this.editForm.getError(key);
				if (formErrors != null && key === 'countryAddressTypeMismatch') {
					hasError = true;
				} else {
					hasError = false;
				}
			});
		}
		return hasError;
	}

	private countryFieldHasError(): boolean {
		return (this.country.invalid && (this.country.dirty || this.country.touched));
	}

	private countyFieldHasError(): boolean {
		return (this.county.invalid && (this.county.dirty || this.county.touched));
	}

	private zipFieldHasError(): boolean {
		return (this.zip.invalid && (this.zip.dirty || this.zip.touched));
	}

	private stateFieldHasError(): boolean {
		return (this.state.invalid && (this.state.dirty || this.state.touched));
	}

	private cityFieldHasError(): boolean {
		return (this.city.invalid && (this.city.dirty || this.city.touched));
	}

	private streetFieldHasError(): boolean {
		return (this.street.invalid && (this.street.dirty || this.street.touched));
	}

	private alertsFieldHasError(): boolean {
		return (this.alerts.invalid && (this.alerts.dirty || this.alerts.touched));
	}

	private emailFieldHasError(): boolean {
		return (this.email.invalid && (this.email.dirty || this.email.touched));
	}

	private mobilePhoneFieldHasError(): boolean {
		return (this.mobilePhone.invalid && (this.mobilePhone.dirty || this.mobilePhone.touched));
	}

	private businessPhoneFieldHasError(): boolean {
		return (this.businessPhone.invalid && (this.businessPhone.dirty || this.businessPhone.touched));
	}

	private homePhoneFieldHasError(): boolean {
		return (this.homePhone.invalid && (this.homePhone.dirty || this.homePhone.touched));
	}

	private commentsFieldHasError(): boolean {
		return (this.comments.invalid && (this.comments.dirty || this.comments.touched));
	}
}
