import { Component, OnInit, ViewEncapsulation} from '@angular/core';
import { Router } from '@angular/router';
import { LogInOutService } from '../../../services/log-in-out/log-in-out.service';

@Component({
	selector: 'log-in',
	templateUrl: './log-in.component.html',
	styleUrls: ['./log-in.component.scss'],
	encapsulation: ViewEncapsulation.None
})
export class LogInComponent implements OnInit {
	public showPassword: boolean = false;
	public customerName: string;
	public copyrightNotice: string;
	public invalidLogin: boolean;
	public userId: string;
	public password: string;
	public loggingIn: boolean;
	public inCordova: boolean;

	constructor(private logInOutService: LogInOutService, private router: Router) {
		this.customerName = '';
		this.copyrightNotice = 'Copyright &copy; Jack Henry & Associates, Inc.';
		this.invalidLogin = false;
		this.userId = '';
		this.password = '';
		this.loggingIn = false;
		this.inCordova = false; // Add code to determine whether we're in Cordova
	}

	ngOnInit(): void {
		// Set focus to username. The input control has autofocus turned on, but
		// that doesn't seem to work when using ui-router to navigate to views.
		window.setTimeout(() => { document.getElementById('inputUser').focus(); }, 500);
	}

	public toggleShowPassword(): void {
		this.showPassword = !this.showPassword;
		window.setTimeout(() => { document.getElementById('showPasswordBtn').blur(); }, 50);
	}

	public LogIn(): void {
		this.loggingIn = true;
		this.logInOutService.LogIn(this.userId, this.password).then(result => this.processLogin(result));
	}

	public logInWithFingerprint(): void {
		console.log('Access Cordova fingerprint API here');
	}

	private processLogin(loginResult: boolean): void {
		this.invalidLogin = !loginResult;

		if (this.invalidLogin) {
			this.loggingIn = false;
		} else {
			this.router.navigate(['Home']);
		}
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
