import { Component, OnInit } from '@angular/core';
import { UntypedFormGroup, UntypedFormControl, Validators } from '@angular/forms';
import { FiData } from '../../../classes/fi-data';


@Component({
	// tslint:disable-next-line:component-selector
	selector: 'master-detail-grid-multiple',
	templateUrl: './master-detail-grid-multiple.component.html',
	styleUrls: ['./master-detail-grid-multiple.component.scss']
})
export class MasterDetailGridMultipleComponent implements OnInit {
	public isSortable: boolean = false;
	public editing: boolean = false;
	public adding: boolean = false;

	public fiList: FiData[] = [];
	public selectedFis: FiData[] = [];

	public popupNotificationText: string = '';
	public showPopupNotification: boolean = false;
	public showDeleteQuestion: boolean = false;

	public editForm: UntypedFormGroup = new UntypedFormGroup({
		'institutionName': new UntypedFormControl('', Validators.required),
		'envVariable': new UntypedFormControl(''),
		'eesUrl': new UntypedFormControl(''),
		'productName': new UntypedFormControl(''),
		'billingId': new UntypedFormControl(''),
		'partnerId': new UntypedFormControl(''),
		'routingNumber': new UntypedFormControl('', [Validators.required, Validators.pattern('^((0[0-9])|(1[0-2])|(2[1-9])|(3[0-2])|(6[1-9])|(7[0-2])|80)([0-9]{7})$')]),
		'notes': new UntypedFormControl('')
	});

	get institutionName(): any { return this.editForm.get('institutionName'); }
	get envVariable(): any { return this.editForm.get('envVariable'); }
	get eesUrl(): any { return this.editForm.get('eesUrl'); }
	get productName(): any { return this.editForm.get('productName'); }
	get billingId(): any { return this.editForm.get('billingId'); }
	get partnerId(): any { return this.editForm.get('partnerId'); }
	get routingNumber(): any { return this.editForm.get('routingNumber'); }
	get notes(): any { return this.editForm.get('notes'); }

	public ngOnInit(): void {

		// START OF DUMMY DATA
		this.fiList = [
			new FiData('Community Bank of Dayton', 'PROD', '', 'iPay', '30050', '72115678910', '101000019', '', '', null, '', null),
			new FiData('Mission Federal Credit Union', 'PROD', '', 'iPay', '12345', '89473627343', '114923756', '', '', null, '', null),
			new FiData('Prosperity Bank', 'PROD', '', 'iPay', '88484', '3882749385', '114882899', '', '', null, '', null),
			new FiData('Golden Bank', 'PROD', '', 'iPay', '33425', '1193829183', '101221333', '', '', null, '', null),
			new FiData('Credit Union of Texas', 'PROD', '', 'iPay', '09904', '1044527384', '120900356', '', '', null, '', null),
			new FiData('Arizona Bank and Trust', 'PROD', '', 'iPay', '98121', '4439420990', '124232873', '', '', null, '', null),
			new FiData('Frost Bank', 'PROD', '', 'iPay', '23221', '54521651221', '112233005', '', '', null, '', null),
			new FiData('InTouch Credit Union', 'PROD', '', 'iPay', '33109', '92312915677', '112872936', '', '', null, '', null),
			new FiData('Neighborhood Credit Union', 'PROD', '', 'iPay', '99033', '4426789011', '102098556', '', '', null, '', null),
			new FiData('Credit Union of South Dakota', 'PROD', '', 'iPay', '44389', '9824682104', '115092091', '', '', null, '', null)
		];

		this.editing = false;
		this.selectedFis = [];
		this.selectedFis.push(this.fiList[0]);
		this.setEditDataFromSelectedFi();

	}

	public addSelectedFi(): void {
		this.editing = false;
		this.setEditDataFromSelectedFi();
	}

	public removeSelectedFi(): void {
		this.editing = false;
		if (this.selectedFis.length > 0) {
			this.setEditDataFromSelectedFi();
		}
	}

	public addSingleSelectedFi($event: any, fi: FiData): void {
		if (!$event.target.classList.contains('p-checkbox-box') && !$event.target.classList.contains('p-checkbox-icon')) {
			this.selectedFis = [];
			this.selectedFis.push(fi);
			window.setTimeout(() => {
				this.selectedFis = [...this.selectedFis];
			}, 10);
		}
	}

	public addMultipleSelectedFi(fi: FiData): void {
		window.setTimeout(() => {
			const selectedIndex: number = this.selectedFis.findIndex((institution) => institution === fi);
			if (selectedIndex >= 0) {
				this.selectedFis.splice(selectedIndex, 1);
				if (this.selectedFis.length > 0) {
					this.setEditDataFromSelectedFi();
				}
			} else {
				this.selectedFis.push(fi);
			}
			this.selectedFis = [...this.selectedFis];
		}, 20);
	}

	public selectAllInstitutions(): void {
		this.selectedFis = [];
		this.fiList.forEach((fi) => {
			this.selectedFis.push(fi);
		});
	}

	public unselectAllInstitutions(): void {
		this.selectedFis = [];
	}

	public startEditing(selectedFi?: string): void {
		if (selectedFi) {
			this.selectedFis = [];
			const index: number = this.fiList.findIndex((fi) => fi.RoutingNumber === selectedFi);
			this.selectedFis.push(this.fiList[index]);
		}

		window.setTimeout(() => {
			this.setEditDataFromSelectedFi();
			this.editing = true;
		}, 10);
	}

	public promptDeleteFi(selectedFi?: string): void {
		this.adding = false;
		this.editing = false;
		if (selectedFi) {
			this.selectedFis = [];
			const index: number = this.fiList.findIndex((fi) => fi.RoutingNumber === selectedFi);
			this.selectedFis.push(this.fiList[index]);
		}

		window.setTimeout(() => {
			this.showDeleteQuestion = true;
		}, 10);
	}

	public startAdding(): void {
		this.selectedFis = [];
		this.setEditDataFromScratch();
		this.adding = true;
		this.editing = true;
	}

	public saveChanges(): void {
		this.editing = false;

		if (!this.adding) {
			this.saveEditData();

			// Display popup notification
			this.popupNotificationText = 'Edited ' + this.institutionName.value;
			this.showPopupNotification = true;
		} else {
			this.fiList.push(
				new FiData(
					this.institutionName.value,
					this.envVariable.value,
					this.eesUrl.value,
					this.productName.value,
					this.billingId.value,
					this.partnerId.value,
					this.routingNumber.value,
					this.notes.value,
					'', null, '', null)
			);
			this.adding = false;

			// Display popup notification
			this.popupNotificationText = 'Added ' + this.institutionName.value;
			this.showPopupNotification = true;

			// Update the selected institution number. We do this after a brief
			// delay to allow the master list time to finish adding the new
			// institution before we select it.
			window.setTimeout(() => {
				const index: number = this.fiList.findIndex((fi) => fi.RoutingNumber === this.routingNumber.value);
				this.selectedFis = [];
				this.selectedFis.push(this.fiList[index]);
			}, 10);
		}

		// Update database here
	}

	public cancelChanges(): void {
		if (this.adding) {
			if (this.fiList.length > 0) {
				this.selectedFis = [];
				this.selectedFis.push(this.fiList[0]);
			}
			this.adding = false;
		}
		this.setEditDataFromSelectedFi();
		this.editing = false;
	}

	public deleteQuestionClosed(dialogResult: string): void {
		if (dialogResult === 'Delete') {
			this.deleteFi();
		}
	}

	private deleteFi(): void {
		let fiName = '';
		this.selectedFis.forEach((selectedFis) => {
			const selectedIndex: number = this.fiList.findIndex((fi) => fi.RoutingNumber === selectedFis.RoutingNumber);
			if (selectedIndex >= 0) {
				if (fiName !== '') {
					fiName = fiName + ' and ' + this.fiList[selectedIndex].InstitutionName;
				} else {
					fiName = this.fiList[selectedIndex].InstitutionName;
				}
				this.fiList.splice(selectedIndex, 1);

				if (this.fiList.length > 0) {
					this.setEditDataFromSelectedFi();
				}

				// Update database here
			}
		});

		// Update fiList so p-table updates rows
		this.fiList = [...this.fiList];

		this.popupNotificationText = 'Deleted ' + fiName;
		this.showPopupNotification = true;
		this.selectedFis = [];

	}

	private setEditDataFromScratch(): void {
		this.editForm.reset();
		this.institutionName.setValue('');
		this.envVariable.setValue('');
		this.eesUrl.setValue('');
		this.productName.setValue('');
		this.billingId.setValue('');
		this.partnerId.setValue('');
		this.routingNumber.setValue('');
		this.notes.setValue('');
	}

	private setEditDataFromSelectedFi(): void {
		this.editForm.reset();
		this.institutionName.setValue(this.selectedFis[0].InstitutionName);
		this.envVariable.setValue(this.selectedFis[0].EnvVariable);
		this.eesUrl.setValue(this.selectedFis[0].EesUrl);
		this.productName.setValue(this.selectedFis[0].ProductName);
		this.billingId.setValue(this.selectedFis[0].BillingId);
		this.partnerId.setValue(this.selectedFis[0].PartnerId);
		this.routingNumber.setValue(this.selectedFis[0].RoutingNumber);
		this.notes.setValue(this.selectedFis[0].Notes);
	}

	private saveEditData(): void {
		this.selectedFis[0].InstitutionName = this.institutionName.value;
		this.selectedFis[0].EnvVariable = this.envVariable.value;
		this.selectedFis[0].EesUrl = this.eesUrl.value;
		this.selectedFis[0].ProductName = this.productName.value;
		this.selectedFis[0].BillingId = this.billingId.value;
		this.selectedFis[0].PartnerId = this.partnerId.value;
		this.selectedFis[0].RoutingNumber = this.routingNumber.value;
		this.selectedFis[0].Notes = this.notes.value;
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
