import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { FiData } from '../../../classes/fi-data';
import { TreeNode } from 'primeng/api';

@Component({
	selector: 'master-detail-tree-view',
	templateUrl: './master-detail-tree-view.component.html',
	styleUrls: ['./master-detail-tree-view.component.scss']
})
export class MasterDetailTreeViewComponent implements OnInit {
	public treeNormal: TreeNode[] = null;
	public treeViewSection: string = '';
	public selectedTreeViewStructure: TreeNode = null;

	constructor(private router: Router) { }

	ngOnInit(): void {

		// Tree data for PrimeNG Treeview
		this.treeNormal = [
			{
				'label': 'Banks',
				'selectable': false,
				'children': [{
						'label': 'Community Bank of Dayton',
						'data' : new FiData('Community Bank of Dayton', 'PROD', '', 'iPay', '30050', '72115678910', '101000019', '', '', null, '', null)
					},
					{
						'label': 'Prosperity Bank',
						'data' : new FiData('Prosperity Bank', 'PROD', '', 'iPay', '88484', '3882749385', '114882899', '', '', null, '', null)
					},
					{
						'label': 'Golden Bank',
						'data' : new FiData('Golden Bank', 'PROD', '', 'iPay', '33425', '1193829183', '101221333', '', '', null, '', null)
					},
					{
						'label': 'Arizona Bank and Trust',
						'data' : new FiData('Arizona Bank and Trust', 'PROD', '', 'iPay', '98121', '4439420990', '124232873', '', '', null, '', null)
					},
					{
						'label': 'Frost Bank',
						'data' : new FiData('Frost Bank', 'PROD', '', 'iPay', '23221', '54521651221', '112233005', '', '', null, '', null)
					}
				]
			},
			{
				'label': 'Credit Unions',
				'selectable': false,
				'children': [
					{
						'label': 'Mission Federal Credit Union',
						'data' : new FiData('Mission Federal Credit Union', 'PROD', '', 'iPay', '12345', '89473627343', '114923756', '', '', null, '', null)
					},
					{
						'label': 'Credit Union of Texas',
						'data' : new FiData('Credit Union of Texas', 'PROD', '', 'iPay', '09904', '1044527384', '120900356', '', '', null, '', null)
					},
					{
						'label': 'InTouch Credit Union',
						'data' : new FiData('InTouch Credit Union', 'PROD', '', 'iPay', '33109', '92312915677', '112872936', '', '', null, '', null)
					},
					{
						'label': 'Neighborhood Credit Union',
						'data' : new FiData('Neighborhood Credit Union', 'PROD', '', 'iPay', '99033', '4426789011', '102098556', '', '', null, '', null)
					},
					{
						'label': 'Credit Union of South Dakota',
						'data' : new FiData('Credit Union of South Dakota', 'PROD', '', 'iPay', '44389', '9824682104', '115092091', '', '', null, '', null)
					}
				]
			}
		];

		// Preselect first node
		this.treeViewSection = this.treeNormal[0].children[0].label;
		this.selectedTreeViewStructure =  this.treeNormal[0].children[0];

		this.treeNormal.forEach(node => {
			this.expandRecursive(node, true);
		});

	}

	// Update display block title on tree selection
	public onTreeNodeSelect(e: any): void {
		this.treeViewSection = e.node.label;
	}

	// Expand Treeview
	private expandRecursive(node: TreeNode, isExpand: boolean): void {
		node.expanded = isExpand;
		if (node.children) {
			node.children.forEach(childNode => {
				this.expandRecursive(childNode, isExpand);
			});
		}
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
