import { Component, OnInit } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';
import { JhaPubSubService, JhaPubSubMessage } from '@jha/jharesponsive/jha-responsive-core';
import { JhaThemingService } from '@jha/jharesponsive/jha-theme';
import { AppComponent } from '../../../app.component';

@Component({
  selector: 'plug-in',
  templateUrl: './plug-in.component.html',
  styleUrls: ['./plug-in.component.scss'],
  providers: [JhaThemingService]
})
export class PlugInComponent implements OnInit {
	private subscription: any = null;
	public baseUrl: string = 'https://jhapluginhosted.z19.web.core.windows.net/';
	public displayedUrl: string = 'jhapluginhosted.z19.web.core.windows.net';
	public hostedUrl: string = '';
	public currentThemeColor: string = 'Pacific';

	constructor(
		private themingService: JhaThemingService,
		private pubSubService: JhaPubSubService,
		public sanitizer: DomSanitizer,
		private appComponent: AppComponent
	) {}

	ngOnInit(): void {
		// Get current theme color
		this.currentThemeColor = JhaThemingService.CurrentThemeColor;

		this.subscription = this.pubSubService.Stream.subscribe(pubSubMessage => this.processMessage(pubSubMessage));

		this.updateUrl();
	}

	public processMessage(pubSubMessage: JhaPubSubMessage): void {
		if (pubSubMessage.message === 'THEME_CHANGED') {
			this.currentThemeColor = JhaThemingService.CurrentThemeColor;
			this.updateUrl();
		}
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

	public updateUrl(): void {
		let url = '';
		url = this.baseUrl + '?themeColor=' + this.currentThemeColor + '&host=responsive';

		this.hostedUrl = this.cleanURL(url);
	}

	private cleanURL(oldURL: string): any {
		return this.sanitizer.bypassSecurityTrustResourceUrl(oldURL);
	}

}
