import { Component, OnInit } from '@angular/core';
import { JhaButtonGroupOption } from '@jha/jharesponsive/jha-buttons';
import { DataService } from '../../../services/data/data.service';
import { TranHistoryDetails } from '../../../classes/tran-history-details';

@Component({
	// tslint:disable-next-line:component-selector
	selector: 'responsive-table',
	templateUrl: './responsive-table.component.html',
	styleUrls: ['./responsive-table.component.scss']
})
export class ResponsiveTableComponent implements OnInit {
	public transactionList: TranHistoryDetails[];
	public sortedTransactionList: TranHistoryDetails[];
	public sortColumn: string;
	public loading: boolean = false;
	public sortDirection: number = 1;
	public sortDescription: string = '';
	public stylingOptions: JhaButtonGroupOption[] = [
		new JhaButtonGroupOption('Minimal', 'Minimal'),
		new JhaButtonGroupOption('Striped', 'Striped')
	];
	public stylingOption: string = 'Minimal';

	constructor(private dataService: DataService) { }

	ngOnInit(): void {
		this.loading = true;
		this.dataService.GetTransactions().then(results => {
			this.loading = false;
			this.transactionList = results;
			this.sortedTransactionList = this.transactionList.slice();
			this.sortHistory('TranDate');
		});
	}

	// Sort the transaction history
	public sortHistory(sortColumn: string): any {
		// Reverse the sort direction
		this.sortDirection *= -1;
		const sortDirection: number = this.sortDirection;

		// Remember the sort column
		this.sortColumn = sortColumn;

		this.sortedTransactionList.sort((itemA: TranHistoryDetails, itemB: TranHistoryDetails) => {
			// Compare ascendening
			let sortResult = 0;
			if (itemA[sortColumn] < itemB[sortColumn]) {
				sortResult = -1;
			} else if (itemA[sortColumn] > itemB[sortColumn]) {
				sortResult = 1;
			}

			// Adjust for sort direction
			sortResult *= sortDirection;

			return sortResult;
		});

		this.updateSortDescription();
	}

	private updateSortDescription(): void {
		this.sortDescription = 'Sorted by ';
		switch (this.sortColumn) {
			case 'TranDate': {
				this.sortDescription += 'Date';
				break;
			}
			case 'EffDate': {
				this.sortDescription += 'Effective';
				break;
			}
			case 'Payee': {
				this.sortDescription += 'Payee';
				break;
			}
			case 'AbsoluteAmount': {
				this.sortDescription += 'Amount';
				break;
			}
			case 'NewBalance': {
				this.sortDescription += 'New Balance';
				break;
			}
		}
		this.sortDescription += (this.sortDirection === -1 ? ' (decreasing) ' : ' (increasing) ');
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
