import { Component } from '@angular/core';
import { Router } from '@angular/router';
import { JhaSessionTimeoutService } from '@jha/jharesponsive/jha-notifications';

@Component({
	selector: 'session-timeout',
	templateUrl: './session-timeout.component.html',
	styleUrls: ['./session-timeout.component.scss']
})
export class SessionTimeoutComponent {
	private sessionTimeoutTimer: any = null;

	constructor(
		private router: Router,
		private jhaSessionTimeoutService: JhaSessionTimeoutService
	) { }


	public logOut(): void {
		// Reset the product-level timer that mocks up a session timeout based on user inactivity
		this.resetSessionTimeout();

		// Code to log out the user goes here...

		// Navigate to the Log In view
		this.router.navigate(['LogIn']);
	}

	public showSessionTimeout(): void {
		// Reset the product-level timer that mocks up a session timeout based on user inactivity
		this.resetSessionTimeout();

		// Set timer that mocks up a session timeout based on user inactivity
		this.sessionTimeoutTimer = setTimeout(() => {
			// Hide any existing session timeout warning already displayed
			this.hideSessionTimeout();

			// Code to log out the user goes here
			this.router.navigate(['InactivityTimeout']);
		}, 20000);

		// Show the session timeout warning with a countdown time of 20 seconds, a callback
		// for the "Extend my Session" button, and a callback for the "Log Out" button
		this.jhaSessionTimeoutService.ShowSessionTimeout(20, () => { this.extendSession(); }, () => { this.logOut(); });
	}

	public showSessionTimeoutNoTimer(): void {
		// Reset the product-level timer that mocks up a session timeout based on user inactivity
		this.resetSessionTimeout();

		// Show the session timeout warning without a countdown time, no "Extend my Session" button,
		// and a callback for the "Log Out" button
		this.jhaSessionTimeoutService.ShowSessionTimeout(0, null, () => { this.logOut(); });
	}

	public hideSessionTimeout(): void {
		// Reset the product-level timer that mocks up a session timeout based on user inactivity
		this.resetSessionTimeout();

		// Manually hide the session timeout warning; note that the warning hides itself when the user
		// presses either the "Extend my Session" or "Log Out" button
		this.jhaSessionTimeoutService.HideSessionTimeout();
	}

	public extendSession(): void {
		// Reset the product-level timer that mocks up a session timeout based on user inactivity
		this.resetSessionTimeout();

		// Alert for this demo
		alert('Session extended');
	}

	private resetSessionTimeout(): void {
		// Reset the product-level timer that mocks up a session timeout based on user inactivity
		clearInterval(this.sessionTimeoutTimer);
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
