import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { UntypedFormGroup, UntypedFormControl, Validators } from '@angular/forms';
import { SelectItem } from 'primeng/api';
import { DataService } from '../../../services/data/data.service';
import { ShippingMethod } from '../../../classes/shipping-method';
import { JhaButtonGroupOption } from '@jha/jharesponsive/jha-buttons';

@Component({
	// tslint:disable-next-line:component-selector
	selector: 'subheader',
	templateUrl: './subheader.component.html',
	styleUrls: ['./subheader.component.scss']
})
export class SubheaderComponent implements OnInit {
	public stateList: SelectItem[] = [];
	public expMonths: SelectItem[] = [];
	public expYears: SelectItem[] = [];
	public shippingMethodList: ShippingMethod[] = [];
	public shippingMethodOptions: JhaButtonGroupOption[] = [];
	public editForm: UntypedFormGroup = new UntypedFormGroup({
		'shippingName': new UntypedFormControl('Nancy Davolio', Validators.required),
		'shippingStreet': new UntypedFormControl('123 Main St', Validators.required),
		'shippingCity': new UntypedFormControl('San Diego', Validators.required),
		'shippingState': new UntypedFormControl('CA', Validators.required),
		'shippingZip': new UntypedFormControl('92110', [Validators.required, Validators.pattern('^[0-9]{5}(?:[0-9]{4})?$')]),
		'billingName': new UntypedFormControl('Nancy Davolio', Validators.required),
		'billingStreet': new UntypedFormControl('123 Main St', Validators.required),
		'billingCity': new UntypedFormControl('San Diego', Validators.required),
		'billingState': new UntypedFormControl('CA', Validators.required),
		'billingZip': new UntypedFormControl('92110', [Validators.required, Validators.pattern('^[0-9]{5}(?:[0-9]{4})?$')]),
	});
	get shippingName(): any { return this.editForm.get('shippingName'); }
	get shippingStreet(): any { return this.editForm.get('shippingStreet'); }
	get shippingCity(): any { return this.editForm.get('shippingCity'); }
	get shippingState(): any { return this.editForm.get('shippingState'); }
	get shippingZip(): any { return this.editForm.get('shippingZip'); }
	get billingName(): any { return this.editForm.get('billingName'); }
	get billingStreet(): any { return this.editForm.get('billingStreet'); }
	get billingCity(): any { return this.editForm.get('billingCity'); }
	get billingState(): any { return this.editForm.get('billingState'); }
	get billingZip(): any { return this.editForm.get('billingZip'); }

	constructor(
		private dataService: DataService
	) {}

	ngOnInit(): void {
		// Get list of states
		this.dataService.GetStateList().then((stateList) => {
			this.stateList = [];
			stateList.forEach((stateOption) => { this.stateList.push({ 'label': stateOption.name, 'value': stateOption.value }); });
		});
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
