import { Component } from '@angular/core';
import { UntypedFormGroup, UntypedFormControl } from '@angular/forms';
import { lastValueFrom } from 'rxjs';
import { JhaUnsavedChangesService } from '@jha/jharesponsive/jha-unsaved-changes';


@Component({
	selector: 'unsaved-changes',
	templateUrl: './unsaved-changes.component.html',
	styleUrls: ['./unsaved-changes.component.scss']
})
export class UnsavedChangesComponent {
	public showUnsavedChangesDialog: boolean = false;

	public editForm: UntypedFormGroup = new UntypedFormGroup({
		'name': new UntypedFormControl('Mary Smith')
	});

	get name(): any { return this.editForm.get('name'); }

	constructor(private unsavedChangesService: JhaUnsavedChangesService) {}

	// canDeactivate for unsaved changes guard
	public async canDeactivate() {
		if (this.editForm.dirty) {
			const promptResult =  this.unsavedChangesService.promptUnsavedChangesDialog(this.editForm.valid);
			
			let promptUserResult = await lastValueFrom(promptResult);
			
			if (promptUserResult === 'Save') {
				// Save changes in database, then return true
				console.log('Saving changes');
				alert('The user chose to save changes, so you should capture the changes and save them to a database here.');
				return true;
			} else if (promptUserResult === 'DontSave') {
				console.log('Dont save changes');
				return true;
			} else if (promptUserResult === 'LoseChanges') {
				console.log('Losing changes');
				return true;
			} else {
				console.log('Returning to editing without leaving page');
				return false;
			};
		} else {
			return true;
		}
	}

	// Save customer changes
	public saveChanges(): void {
		alert('All fields were successfully validated, so your app would save change and reset the form to be pristine.');
		// Update database here
		this.editForm.markAsPristine();
		this.editForm.markAsUntouched();
	}

	public cancelChanges(): void {
		if (this.editForm.dirty) {
			this.actuallyCancelChanges();
		}
	}

	public actuallyCancelChanges(): void {
		// reset form
		this.name.setValue('Mary Smith');
		this.editForm.markAsPristine();
	}
}
