﻿import { Injectable } from '@angular/core';

import { CustomerDetails } from '../../components/record-detail/customer-details';

@Injectable()
export class CustomerDetailsService {
	public getCustomerDetailInfo(): Promise<CustomerDetails> {
		// Dummy up customer detail info here
		const customerDetailInfo = new CustomerDetails(
			'Nancy Davolio',
			true,
			8.75,
			111223333,
			new Date(1976, 3, 1),
			0,
			45000.00,
			false,
			new Date(2016, 0, 15, 14, 18),
			'123 Main St',
			'Springfield',
			'MO',
			658071234,
			'USA',
			'Domestic',
			'417-888-9631',
			'417-541-1980',
			'417-626-8827',
			'carrie.smith@gmail.com',
			{},
			[]
		);

		// Recalculate the age from the birthdate
		let calculatedAge = 0;
		if (customerDetailInfo.birthdate != null) {
			const fieldDate = customerDetailInfo.birthdate;
			const today = new Date();
			calculatedAge = today.getFullYear() - fieldDate.getFullYear();
			if (today.getMonth() < fieldDate.getMonth() || (today.getMonth() === fieldDate.getMonth() && today.getDate() < fieldDate.getDate())) {
				calculatedAge--;
			}
		}
		customerDetailInfo.age = calculatedAge;

		return new Promise((resolve) => {
			setTimeout(() => resolve(customerDetailInfo), 500);
		});
	}

	// Post customer detail info
	public postCustomerDetailInfo(customerInfo: CustomerDetails): Promise<boolean> {
		customerInfo.recordLevelValidationErrors = [];
		customerInfo.fieldLevelValidationErrors = {};

		// Fake record-level validations
		if ((customerInfo.country === 'USA' && customerInfo.addressType === 'International') ||
			(customerInfo.country !== 'USA' && customerInfo.addressType === 'Domestic')) {
			customerInfo.recordLevelValidationErrors.push(customerInfo.country + ' is an invalid country for ' + customerInfo.addressType + ' address type');
		}

		// Fake field-level validations
		this.fakeValidateCustomerDetailField('Name', customerInfo);
		this.fakeValidateCustomerDetailField('InterestRate', customerInfo);

		// Return value is an error flag, so check for errors
		let errorFlag: boolean = false;
		if (customerInfo.recordLevelValidationErrors.length > 0 ||
			Object.keys(customerInfo.fieldLevelValidationErrors).length > 0) {
			errorFlag = true;
		}

		return new Promise((resolve) => {
			setTimeout(() => resolve(errorFlag), 500);
		});
	}

	// Fake field-level validation; these would typically be done by the back-end service
	public fakeValidateCustomerDetailField(fieldId: string, customerInfo: CustomerDetails): void {
		let validationError = '';

		if (fieldId === 'Name') {
			if (!customerInfo.name) {
				validationError = 'Please fill out this field.';
			} else if (customerInfo.name && customerInfo.name.toLowerCase() === 'jimbo') {
				validationError = 'This name is not allowed.';
			}
		} else if (fieldId === 'InterestRate') {
			if (!Boolean(customerInfo.interestRate % 2)) {
				validationError = 'Value must be odd.';
			}
		}

		if (validationError) {
			customerInfo.fieldLevelValidationErrors[fieldId] = validationError;
		}
	}
}
