﻿import { Injectable } from '@angular/core';
import { JhaPubSubService, JhaListOption } from '@jha/jharesponsive/jha-responsive-core';
import { User } from '../../classes/user';
import { Account } from '../../classes/account';
import { AlertGroup } from '../../classes/alert-group';
import { Employee } from '../../classes/employee';
import { TranHistoryDetails } from '../../classes/tran-history-details';
import { CustomerDetails } from '../../classes/customer-details';
import { TreeNode } from 'primeng/api';
import { SelectItem } from 'primeng/api';
import { Observable, of } from 'rxjs';

@Injectable()
export class DataService {
	private employees: Employee[];

	constructor(private pubSubService: JhaPubSubService) { }

	// Get list of all users
	public GetAllUsers(): Promise<User[]> {
		// Dummy data logic
		const users: User[] =
			[
				new User('300', 'Donny', 'Conway', 'dconway'),
				new User('301', 'James', 'Asher', 'jasher'),
				new User('302', 'Ken', 'Schultz', 'kschultz'),
				new User('303', 'Jason', 'Arndt', 'jarndt'),
				new User('304', 'Luke', 'Killinger', 'lkilling'),
				new User('305', 'Jim', 'Honeycutt', 'jhoneycu')
			];
		return new Promise((resolve) => {
			setTimeout(() => resolve(users), 1000);
		});
	}

	// Get list of accounts for the current user
	public GetAccountsForUser(): Promise<Account[]> {
		// Dummy data logic
		const accounts: Account[] =
			[
				new Account('100', 'S', 176181),
				new Account('200', 'D', 51832)
			];
		return new Promise((resolve) => {
			setTimeout(() => resolve(accounts), 1000);
		});
	}

	// Get list of transactions for the current user
	public GetTransactions(): Promise<Array<TranHistoryDetails>> {
		const tranHistoryData = new Array<TranHistoryDetails>();

		// Dummy up the transaction history data
		const count: number = 10;
		const tranDate: Date = new Date();
		let newBalance: number = (Math.floor(Math.random() * 100000) / 100) + 250;
		const payees: Array<string> =
			[
				'Sprouts',
				'Whole Foods',
				'Costco',
				'Bloomingdales',
				'ITunes',
				'Netflix',
				'Tender Greens',
				'GNC',
				'Baja Betty\'s',
				'Bow Wow Beauty Shop',
				'Flick\'s',
				'Chevron',
				'Rich\'s',
				'Babycakes',
				'24 Hour Fitness'
			];
		let payee: string;
		let amount: number;

		for (let i = 0; i < count && newBalance > 0; i++) {
			// Get transaction date
			tranDate.setDate(tranDate.getDate() - Math.floor(Math.random() * 4));

			// Effective date is Monday if tran date is a weekend; otherwise the same as tran date
			let effDateOffset = 0;
			if (tranDate.getDay() === 6) {
				effDateOffset = 2;  // Saturday
			} else if (tranDate.getDay() === 0) {
				effDateOffset = 1;  // Sunday
			}
			const effDate = new Date();
			effDate.setDate(tranDate.getDate() + effDateOffset);

			if (newBalance < 100) {
				// Kick in a deposit if we're getting too low
				payee = 'Deposit';
				amount = (Math.floor(Math.random() * 10000) / 100) + 500;
			} else {
				payee = payees[Math.floor(Math.random() * payees.length)];
				amount = (Math.floor(Math.random() * -5000) / 100) - 10;
				if (-amount > newBalance) {
					amount = -newBalance;
				}
			}
			newBalance += amount;

			tranHistoryData.push(new TranHistoryDetails(
				i + 1,
				new Date(tranDate.toDateString()),
				new Date(effDate.toDateString()),
				payee,
				amount,
				Math.abs(amount),
				newBalance
			));
		}

		return new Promise((resolve) => {
			setTimeout(() => resolve(tranHistoryData), 1000);
		});
	}

	public SendMoney(receipient: string, accountId: string, amount: number, comment: string): Promise<boolean> {
		// Dummy data logic
		return new Promise((resolve) => { setTimeout(() => resolve(true), 1000); });
	}

	public GetAccountTypeList(): Promise<SelectItem[]> {
		const accountTypeList: SelectItem[] =
			[
				{ label: 'Checking', value: 'Checking' },
				{ label: 'Business Checking', value: 'Business Checking' },
				{ label: 'Student Checking', value: 'Student Checking' },
				{ label: 'Savings', value: 'Savings' },
				{ label: 'Money Market', value: 'Money Market' },
				{ label: 'CD', value: 'CD' },
				{ label: 'IRA', value: 'IRA' }
			];
		return new Promise((resolve) => {
			setTimeout(() => resolve(accountTypeList), 10);
		});
	}

	public GetStateList(): Promise<JhaListOption[]> {
		const stateList: JhaListOption[] =
			[
				new JhaListOption('Alabama', 'AL'),
				new JhaListOption('Alaska', 'AK'),
				new JhaListOption('Arizona', 'AZ'),
				new JhaListOption('Arkansas', 'AR'),
				new JhaListOption('California', 'CA'),
				new JhaListOption('Colorado', 'CO'),
				new JhaListOption('Connecticut', 'CT'),
				new JhaListOption('Delaware', 'DE'),
				new JhaListOption('Florida', 'FL'),
				new JhaListOption('Georgia', 'GA'),
				new JhaListOption('Hawaii', 'HI'),
				new JhaListOption('Idaho', 'ID'),
				new JhaListOption('Illinois', 'IL'),
				new JhaListOption('Indiana', 'IN'),
				new JhaListOption('Iowa', 'IA'),
				new JhaListOption('Kansas', 'KS'),
				new JhaListOption('Kentucky', 'KY'),
				new JhaListOption('Louisiana', 'LA'),
				new JhaListOption('Maine', 'ME'),
				new JhaListOption('Maryland', 'MD'),
				new JhaListOption('Massachusetts', 'MA'),
				new JhaListOption('Michigan', 'MI'),
				new JhaListOption('Minnesota', 'MN'),
				new JhaListOption('Mississippi', 'MS'),
				new JhaListOption('Missouri', 'MO'),
				new JhaListOption('Montana', 'MT'),
				new JhaListOption('Nebraska', 'NE'),
				new JhaListOption('Nevada', 'NV'),
				new JhaListOption('New Hampshire', 'NH'),
				new JhaListOption('New Jersey', 'NJ'),
				new JhaListOption('New Mexico', 'NM'),
				new JhaListOption('New York', 'NY'),
				new JhaListOption('North Carolina', 'NC'),
				new JhaListOption('North Dakota', 'ND'),
				new JhaListOption('Ohio', 'OH'),
				new JhaListOption('Oklahoma', 'OK'),
				new JhaListOption('Oregon', 'OR'),
				new JhaListOption('Pennsylvania', 'PA'),
				new JhaListOption('Rhode Island', 'RI'),
				new JhaListOption('South Carolina', 'SC'),
				new JhaListOption('South Dakota', 'SD'),
				new JhaListOption('Tennessee', 'TN'),
				new JhaListOption('Texas', 'TX'),
				new JhaListOption('Utah', 'UT'),
				new JhaListOption('Vermont', 'VT'),
				new JhaListOption('Virginia', 'VA'),
				new JhaListOption('Washington', 'WA'),
				new JhaListOption('West Virginia', 'WV'),
				new JhaListOption('Wisconsin', 'WI'),
				new JhaListOption('Wyoming', 'WY')
			];
		return new Promise((resolve) => {
			setTimeout(() => resolve(stateList), 10);
		});
	}

	public GetCountyList(): Promise<SelectItem[]> {
		const countyList: SelectItem[] =
			[
				{ label: 'Kings', value: 'Kings' },
				{ label: 'Queens', value: 'Queens' },
				{ label: 'New York', value: 'New York' },
				{ label: 'Suffolk', value: 'Suffolk' },
				{ label: 'Bronx', value: 'Bronx' },
				{ label: 'Nassau', value: 'Nassau' },
				{ label: 'Westchester', value: 'Westchester' },
				{ label: 'Erie', value: 'Erie' },
				{ label: 'Monroe', value: 'Monroe' },
				{ label: 'Richmond', value: 'Richmond' },
				{ label: 'Onondaga', value: 'Onondaga' },
				{ label: 'Orange', value: 'Orange' },
				{ label: 'Rockland', value: 'Rockland' },
				{ label: 'Albany', value: 'Albany' },
				{ label: 'Dutchess', value: 'Dutchess' },
				{ label: 'Saratoga', value: 'Saratoga' },
				{ label: 'Oneida', value: 'Oneida' },
				{ label: 'Niagara', value: 'Niagara' },
				{ label: 'Broome', value: 'Broome' },
				{ label: 'Ulster', value: 'Ulster' },
				{ label: 'Rensselaer', value: 'Rensselaer' },
				{ label: 'Schenectady', value: 'Schenectady' },
				{ label: 'Chautauqua', value: 'Chautauqua' },
				{ label: 'Oswego', value: 'Oswego' },
				{ label: 'Jefferson', value: 'Jefferson' },
				{ label: 'Ontario', value: 'Ontario' },
				{ label: 'St. Lawrence', value: 'St. Lawrence' },
				{ label: 'Tompkins', value: 'Tompkins' },
				{ label: 'Putnam', value: 'Putnam' },
				{ label: 'Steuben', value: 'Steuben' },
				{ label: 'Wayne', value: 'Wayne' },
				{ label: 'Chemung', value: 'Chemung' },
				{ label: 'Clinton', value: 'Clinton' },
				{ label: 'Cayuga', value: 'Cayuga' },
				{ label: 'Cattaraugus', value: 'Cattaraugus' },
				{ label: 'Sullivan', value: 'Sullivan' },
				{ label: 'Madison', value: 'Madison' },
				{ label: 'Warren', value: 'Warren' },
				{ label: 'Livingston', value: 'Livingston' },
				{ label: 'Herkimer', value: 'Herkimer' },
				{ label: 'Washington', value: 'Washington' },
				{ label: 'Columbia', value: 'Columbia' },
				{ label: 'Otsego', value: 'Otsego' },
				{ label: 'Genesee', value: 'Genesee' },
				{ label: 'Fulton', value: 'Fulton' },
				{ label: 'Franklin', value: 'Franklin' },
				{ label: 'Montgomery', value: 'Montgomery' },
				{ label: 'Tioga', value: 'Tioga' },
				{ label: 'Cortland', value: 'Cortland' },
				{ label: 'Chenango', value: 'Chenango' },
				{ label: 'Greene', value: 'Greene' },
				{ label: 'Allegany', value: 'Allegany' },
				{ label: 'Delaware', value: 'Delaware' },
				{ label: 'Orleans', value: 'Orleans' },
				{ label: 'Wyoming', value: 'Wyoming' },
				{ label: 'Essex', value: 'Essex' },
				{ label: 'Seneca', value: 'Seneca' },
				{ label: 'Schoharie', value: 'Schoharie' },
				{ label: 'Lewis', value: 'Lewis' },
				{ label: 'Yates', value: 'Yates' },
				{ label: 'Schuyler', value: 'Schuyler' },
				{ label: 'Hamilton', value: 'Hamilton' }

			];
		return new Promise((resolve) => {
			setTimeout(() => resolve(countyList), 10);
		});
	}

	public GetCountryList(): Promise<string[]> {
		const countryList: string[] =
			[
				'Afghanistan',
				'Albania',
				'Algeria',
				'Andorra',
				'Angola',
				'Antigua and Barbuda',
				'Argentina',
				'Armenia',
				'Australia',
				'Austria',
				'Azerbaijan',
				'Bahamas',
				'Bahrain',
				'Bangladesh',
				'Barbados',
				'Belarus',
				'Belgium',
				'Belize',
				'Benin',
				'Bhutan',
				'Bolivia',
				'Bosnia and Herzegovina',
				'Botswana',
				'Brazil',
				'Brunei',
				'Bulgaria',
				'Burkina Faso',
				'Burundi',
				'Cabo Verde',
				'Cambodia',
				'Cameroon',
				'Canada',
				'Central African Republic',
				'Chad',
				'Chile',
				'China',
				'Colombia',
				'Comoros',
				'Republic of the Congo',
				'Democratic Republic of theCongo',
				'Costa Rica',
				'Cote d\'Ivoire',
				'Croatia',
				'Cuba',
				'Cyprus',
				'Czech Republic',
				'Denmark',
				'Djibouti',
				'Dominica',
				'Dominican Republic',
				'Ecuador',
				'Egypt',
				'El Salvador',
				'Equatorial Guinea',
				'Eritrea',
				'Estonia',
				'Ethiopia',
				'Fiji',
				'Finland',
				'France',
				'Gabon',
				'Gambia',
				'Georgia',
				'Germany',
				'Ghana',
				'Greece',
				'Grenada',
				'Guatemala',
				'Guinea',
				'Guinea-Bissau',
				'Guyana',
				'Haiti',
				'Honduras',
				'Hungary',
				'Iceland',
				'India',
				'Indonesia',
				'Iran',
				'Iraq',
				'Ireland',
				'Israel',
				'Italy',
				'Jamaica',
				'Japan',
				'Jordan',
				'Kazakhstan',
				'Kenya',
				'Kiribati',
				'Kosovo',
				'Kuwait',
				'Kyrgyzstan',
				'Laos',
				'Latvia',
				'Lebanon',
				'Lesotho',
				'Liberia',
				'Libya',
				'Liechtenstein',
				'Lithuania',
				'Luxembourg',
				'Macedonia',
				'Madagascar',
				'Malawi',
				'Malaysia',
				'Maldives',
				'Mali',
				'Malta',
				'Marshall Islands',
				'Mauritania',
				'Mauritius',
				'Mexico',
				'Micronesia',
				'Moldova',
				'Monaco',
				'Mongolia',
				'Montenegro',
				'Morocco',
				'Mozambique',
				'Myanmar',
				'Namibia',
				'Nauru',
				'Nepal',
				'Netherlands',
				'New Zealand',
				'Nicaragua',
				'Niger',
				'Nigeria',
				'North Korea',
				'Norway',
				'Oman',
				'Pakistan',
				'Palau',
				'Palestine',
				'Panama',
				'Papua New Guinea',
				'Paraguay',
				'Peru',
				'Philippines',
				'Poland',
				'Portugal',
				'Qatar',
				'Romania',
				'Russia',
				'Rwanda',
				'St. Kitts and Nevis',
				'St. Lucia',
				'St. Vincent and The Grenadines',
				'Samoa',
				'San Marino',
				'Sao Tome and Principe',
				'Saudi Arabia',
				'Senegal',
				'Serbia',
				'Seychelles',
				'Sierra Leone',
				'Singapore',
				'Slovakia',
				'Slovenia',
				'Solomon Islands',
				'Somalia',
				'South Africa',
				'South Korea',
				'South Sudan',
				'Spain',
				'Sri Lanka',
				'Sudan',
				'Suriname',
				'Swaziland',
				'Sweden',
				'Switzerland',
				'Syria',
				'Taiwan',
				'Tajikistan',
				'Tanzania',
				'Thailand',
				'Timor-Leste',
				'Togo',
				'Tonga',
				'Trinidad and Tobago',
				'Tunisia',
				'Turkey',
				'Turkmenistan',
				'Tuvalu',
				'Uganda',
				'Ukraine',
				'United Arab Emirates',
				'UK',
				'USA',
				'Uruguay',
				'Uzbekistan',
				'Vanuatu',
				'Vatican City',
				'Venezuela',
				'Vietnam',
				'Yemen',
				'Zambia',
				'Zimbabwe'
			];
		return new Promise((resolve) => {
			setTimeout(() => resolve(countryList), 10);
		});
	}

	public getAlertTypeList(): Promise<AlertGroup[]> {
		const alertTypeGroups: AlertGroup[] = [
			{
				name: 'Security',
				alert: [
					{category: 'security-alerts', description: 'Security Alerts'},
					{category: 'security-other', description: 'Other Alerts'}
				]
			},
			{
				name: 'Checking Alerts',
				alert: [
					{category: 'check-balancing', description: 'Balancing & Spending'},
					{category: 'check-payments', description: 'Payments'},
					{category: 'check-security', description: 'Security Alerts'}
				]
			},
			{
				name: 'Savings Alerts',
				alert: [
					{category: 'savings-balancing', description: 'Balancing & Spending'},
					{category: 'savings-payments', description: 'Payments'},
					{category: 'savings-security', description: 'Security Alerts'}
				]
			},
			{
				name: 'Retirement Alerts',
				disabled: true,
				alert: [
					{category: 'retirement-balancing', description: 'Balancing & Spending', disabled: true},
					{category: 'retirement-payments', description: 'Payments', disabled: true},
					{category: 'retirement-security', description: 'Security Alerts', disabled: true}
				]
			},
			{
				name: 'Delivery Preferences',
				alert: [
					{category: 'delivery-email', description: 'Email'},
					{category: 'delivery-text', description: 'Text Message'},
					{category: 'delivery-phone', description: 'Phone Call'}
				]
			}
		];
		return new Promise((resolve) => {
			setTimeout(() => resolve(alertTypeGroups), 10);
		});
	}

	public getCustomerDetailInfo(): Promise<CustomerDetails> {
		// Dummy up customer detail info here
		const customerDetailInfo = new CustomerDetails(
			'Nancy Davolio',
			true,
			8.75,
			111223333,
			new Date(1976, 3, 1),
			0,
			45000.00,
			false,
			['Checking', 'IRA'],
			'Silver',
			'Customer Account',
			new Date(2016, 0, 15, 14, 18),
			'123 Main St',
			'Springfield',
			'MO',
			658071234,
			'Nassau',
			'USA',
			'Domestic',
			'417-888-9631',
			'417-541-1980',
			'417-626-8827',
			'carrie.smith@gmail.com',
			['security-other', 'check-balancing'],
			'',
			{},
			[]
		);

		// Recalculate the age from the birthdate
		let calculatedAge = 0;
		if (customerDetailInfo.birthdate != null) {
			const fieldDate = customerDetailInfo.birthdate;
			const today = new Date();
			calculatedAge = today.getFullYear() - fieldDate.getFullYear();
			if (today.getMonth() < fieldDate.getMonth() ||
				(today.getMonth() === fieldDate.getMonth() && today.getDate() < fieldDate.getDate())) {
				calculatedAge--;
			}
		}
		customerDetailInfo.age = calculatedAge;

		return new Promise((resolve) => {
			setTimeout(() => resolve(customerDetailInfo), 500);
		});
	}

	// Post customer detail info
	public postCustomerDetailInfo(customerInfo: CustomerDetails): Promise<boolean> {
		customerInfo.recordLevelValidationErrors = [];
		customerInfo.fieldLevelValidationErrors = {};

		// Fake record-level validations
		if ((customerInfo.country === 'USA' && customerInfo.addressType === 'International') ||
			(customerInfo.country !== 'USA' && customerInfo.addressType === 'Domestic')) {
			customerInfo.recordLevelValidationErrors.push(customerInfo.country + ' is an invalid country for ' + customerInfo.addressType + ' address type');
		}

		// Fake field-level validations
		this.fakeValidateCustomerDetailField('Name', customerInfo);
		this.fakeValidateCustomerDetailField('InterestRate', customerInfo);

		// Return value is an error flag, so check for errors
		let errorFlag: boolean = false;
		if (customerInfo.recordLevelValidationErrors.length > 0 ||
			Object.keys(customerInfo.fieldLevelValidationErrors).length > 0) {
			errorFlag = true;
		}

		return new Promise((resolve) => {
			setTimeout(() => resolve(errorFlag), 500);
		});
	}

	// Fake field-level validation; these would typically be done by the back-end service
	public fakeValidateCustomerDetailField(fieldId: string, customerInfo: CustomerDetails): void {
		let validationError = '';

		if (fieldId === 'Name') {
			if (!customerInfo.name) {
				validationError = 'Please fill out this field.';
			} else if (customerInfo.name && customerInfo.name.toLowerCase() === 'jimbo') {
				validationError = 'This name is not allowed.';
			}
		} else if (fieldId === 'InterestRate') {
			if (!Boolean(customerInfo.interestRate % 2)) {
				validationError = 'Value must be odd.';
			}
		}

		if (validationError) {
			customerInfo.fieldLevelValidationErrors[fieldId] = validationError;
		}
	}

	// Get count of employees
	public GetEmployeeTotalCount(): Promise<number> {
		this.InitializeEmployees();

		return new Promise((resolve) => {
			setTimeout(() => resolve(this.employees.length), 10);
		});
	}

	// Get list of all users
	public GetEmployees(startIndex: number, count: number): Promise<Employee[]> {
		let endIndex: number = startIndex + count;
		if (endIndex > this.employees.length) {
			endIndex = this.employees.length;
		}
		const employeeDataset: Employee[] = this.employees.slice(startIndex, endIndex);
		return new Promise((resolve) => {
			setTimeout(() => resolve(employeeDataset), 600);
		});
	}

	private InitializeEmployees(): void {
		// Dummy data logic
		this.employees = [];
		const count: number = Math.floor(Math.random() * 50) + 50;
		const maleFirstNames: Array<string> =
			[
				'John',
				'Jim',
				'Michael',
				'Steve',
				'Robert',
				'Jarod',
				'Chris',
				'Matt',
				'Greg',
				'Ryan'
			];
		const femaleFirstNames: Array<string> =
			[
				'Linda',
				'Sue',
				'Mary',
				'Amy',
				'Sheila',
				'Brittany',
				'Joan',
				'Jessica',
				'Mindy',
				'Martha'
			];
		const lastNames: Array<string> =
			[
				'Smith',
				'Jones',
				'Johnson',
				'Miller',
				'Brown',
				'Martinez',
				'Tucker',
				'Underwood',
				'Martin',
				'Broadwell'
			];
		let malePhotoIndex: number = 1;
		let femalePhotoIndex: number = 1;

		let employeeId: number = 500;
		for (let i = 0; i < count; i++) {
			const sex: string = Math.floor(Math.random() * 2) === 0 ? 'M' : 'F';
			let name: string = '';
			let photoId: string = '';
			if (sex === 'M') {
				const firstName: string = maleFirstNames[Math.floor(Math.random() * maleFirstNames.length)];
				const lastName: string = lastNames[Math.floor(Math.random() * lastNames.length)];
				name = firstName + ' ' + lastName;
				photoId = malePhotoIndex.toString();
				malePhotoIndex++;
				if (malePhotoIndex > 10) {
					malePhotoIndex = 1;
				}
				if (photoId.length < 2) {
					photoId = '0' + photoId;
				}
				photoId = 'm' + photoId + '.jpg';
			} else {
				const firstName: string = femaleFirstNames[Math.floor(Math.random() * femaleFirstNames.length)];
				const lastName: string = lastNames[Math.floor(Math.random() * lastNames.length)];
				name = firstName + ' ' + lastName;
				photoId = femalePhotoIndex.toString();
				femalePhotoIndex++;
				if (femalePhotoIndex > 10) {
					femalePhotoIndex = 1;
				}
				if (photoId.length < 2) {
					photoId = '0' + photoId;
				}
				photoId = 'f' + photoId + '.jpg';
			}

			this.employees.push(new Employee((employeeId).toString(), name, sex, photoId));
			employeeId += Math.floor(Math.random() * 100);
		}
	}

	// Mock file upload service
	public postFile(filesToUpload: any[]): Observable<any> {
		console.log(filesToUpload);
		return of(filesToUpload);
	}

	public getFileTypeList(): Promise<SelectItem[]> {
		const fileTypeList: SelectItem[] =
			[
				{ label: 'Application', value: 'Application' },
				{ label: 'Folder', value: 'Folder' },
				{ label: 'Link', value: 'Link' },
				{ label: 'PDF', value: 'PDF' },
				{ label: 'Picture', value: 'Picture' },
				{ label: 'Resume', value: 'Resume' },
				{ label: 'Text', value: 'Text' },
				{ label: 'Video', value: 'Video' },
				{ label: 'Zip', value: 'Zip' }
			];
		return new Promise((resolve) => {
			setTimeout(() => resolve(fileTypeList), 10);
		});
	}

	public getFileSystemCols(): Promise<SelectItem[]> {
		const cols: any[] =
			[
				{ field: 'name', header: 'Name' },
				{ field: 'size', header: 'Size' },
				{ field: 'type', header: 'Type' }
			];
		return new Promise((resolve) => {
			setTimeout(() => resolve(cols), 10);
		});
	}

	public getFileSystem(): Promise<TreeNode[]> {

		const files: TreeNode[] =
		[
			{
				'data': {
					'name': 'Applications',
					'size': '200 MB',
					'type': 'Folder'
				},
				'children': [
					{
						'data': {
							'name': 'Angular',
							'size': '25 MB',
							'type': 'Folder'
						},
						'children': [
							{
								'data': {
									'name': 'angular.app',
									'size': '10 MB',
									'type': 'Application'
								}
							},
							{
								'data': {
									'name': 'cli.app',
									'size': '10 MB',
									'type': 'Application'
								}
							},
							{
								'data': {
									'name': 'mobile.app',
									'size': '5 MB',
									'type': 'Application'
								}
							}
						]
					},
					{
						'data': {
							'name': 'editor.app',
							'size': '25 MB',
							'type': 'Application'
						}
					},
					{
						'data': {
							'name': 'settings.app',
							'size': '50 MB',
							'type': 'Application'
						}
					}
				]
			},
			{
				'data': {
					'name': 'Cloud',
					'size': '20 MB',
					'type': 'Folder'
				},
				'children': [
					{
						'data': {
							'name': 'backup-1.zip',
							'size': '10 MB',
							'type': 'Zip'
						}
					},
					{
						'data': {
							'name': 'backup-2.zip',
							'size': '10 MB',
							'type': 'Zip'
						}
					}
				]
			},
			{
				'data': {
					'name': 'Desktop',
					'size': '150 KB',
					'type': 'Folder'
				},
				'children': [
					{
						'data': {
							'name': 'note-meeting.txt',
							'size': '50 KB',
							'type': 'Text'
						}
					},
					{
						'data': {
							'name': 'note-todo.txt',
							'size': '100 KB',
							'type': 'Text'
						}
					}
				]
			},
			{
				'data': {
					'name': 'Documents',
					'size': '75 KB',
					'type': 'Folder'
				},
				'children': [
					{
						'data': {
							'name': 'Work',
							'size': '55 KB',
							'type': 'Folder'
						},
						'children': [
							{
								'data': {
									'name': 'Expenses.doc',
									'size': '30 KB',
									'type': 'Document'
								}
							},
							{
								'data': {
									'name': 'Resume.doc',
									'size': '25 KB',
									'type': 'Resume'
								}
							}
						]
					},
					{
						'data': {
							'name': 'Home',
							'size': '20 KB',
							'type': 'Folder'
						},
						'children': [
							{
								'data': {
									'name': 'Invoices',
									'size': '20 KB',
									'type': 'Text'
								}
							}
						]
					}
				]
			},
			{
				'data': {
					'name': 'Downloads',
					'size': '25 MB',
					'type': 'Folder'
				},
				'children': [
					{
						'data': {
							'name': 'Spanish',
							'size': '10 MB',
							'type': 'Folder'
						},
						'children': [
							{
								'data': {
									'name': 'tutorial-a1.txt',
									'size': '5 MB',
									'type': 'Text'
								}
							},
							{
								'data': {
									'name': 'tutorial-a2.txt',
									'size': '5 MB',
									'type': 'Text'
								}
							}
						]
					},
					{
						'data': {
							'name': 'Travel',
							'size': '15 MB',
							'type': 'Text'
						},
						'children': [
							{
								'data': {
									'name': 'Hotel.pdf',
									'size': '10 MB',
									'type': 'PDF'
								}
							},
							{
								'data': {
									'name': 'Flight.pdf',
									'size': '5 MB',
									'type': 'PDF'
								}
							}
						]
					}
				]
			},
			{
				'data': {
					'name': 'Main',
					'size': '50 MB',
					'type': 'Folder'
				},
				'children': [
					{
						'data': {
							'name': 'bin',
							'size': '50 KB',
							'type': 'Link'
						}
					},
					{
						'data': {
							'name': 'etc',
							'size': '100 KB',
							'type': 'Link'
						}
					},
					{
						'data': {
							'name': 'var',
							'size': '100 KB',
							'type': 'Link'
						}
					}
				]
			},
			{
				'data': {
					'name': 'Other',
					'size': '5 MB',
					'type': 'Folder'
				},
				'children': [
					{
						'data': {
							'name': 'todo.txt',
							'size': '3 MB',
							'type': 'Text'
						}
					},
					{
						'data': {
							'name': 'logo.png',
							'size': '2 MB',
							'type': 'Picture'
						}
					}
				]
			},
			{
				'data': {
					'name': 'Pictures',
					'size': '150 KB',
					'type': 'Folder'
				},
				'children': [
					{
						'data': {
							'name': 'barcelona.jpg',
							'size': '90 KB',
							'type': 'Picture'
						}
					},
					{
						'data': {
							'name': 'primeng.png',
							'size': '30 KB',
							'type': 'Picture'
						}
					},
					{
						'data': {
							'name': 'prime.jpg',
							'size': '30 KB',
							'type': 'Picture'
						}
					}
				]
			},
			{
				'data': {
					'name': 'Videos',
					'size': '1500 MB',
					'type': 'Folder'
				},
				'children': [
					{
						'data': {
							'name': 'primefaces.mkv',
							'size': '1000 MB',
							'type': 'Video'
						}
					},
					{
						'data': {
							'name': 'intro.avi',
							'size': '500 MB',
							'type': 'Video'
						}
					}
				]
			}
		];

		return new Promise((resolve) => {
			setTimeout(() => resolve(files), 10);
		});
	}
}
