﻿import { Injectable } from '@angular/core';

import { JhaPubSubService, JhaPubSubMessage  } from '@jha/jharesponsive/jha-responsive-core';

import { User } from '../../classes/user';

@Injectable()
export class LogInOutService {
	private loggedIn: boolean = false;
	private loggedInUser: User;

	constructor(private pubSubService: JhaPubSubService) { }

	// Is the user currently logged in?
	public IsLoggedIn(): boolean {
		return this.loggedIn;
	}

	// Get the logged-in user
	public GetLoggedInUser(): User {
		return this.loggedInUser;
	}

	// Log in
	public LogIn(userId: string, password: string): Promise<boolean> {
		// Dummy login logic
		if (userId.trim() !== '' && password.trim() !== '') {
			this.loggedIn = true;
			this.loggedInUser = new User('305', 'Jim', 'Honeycutt', 'jhoneycu');
		} else {
			this.loggedIn = false;
		}

		// Publish message that user is logged in
		if (this.loggedIn) {
			this.pubSubService.Stream.next(new JhaPubSubMessage('LOGGEDIN', null));
		}

		return new Promise((resolve) => {
			setTimeout(() => resolve(this.loggedIn), 1000);
		});
	}

	// Log out
	public LogOut(): void {
		this.loggedIn = false;

		// Publish message that user is logged out
		this.pubSubService.Stream.next(new JhaPubSubMessage('LOGGEDOUT', null));
	}
}
